@@ -237,9 +237,9 @@ protected:
    virtual void procElsIfCondition() = 0;
    virtual void procWhileStatement() = 0;
    virtual void procWhileCondition() = 0;
-   virtual void wcConditionAnd() = 0;
-   virtual void wcConditionOr() = 0;
-   virtual void wcConditionPredicate() = 0;
+   virtual void conditionOr() = 0;
+   virtual void conditionAnd() = 0;
+   virtual void conditionPredicate() = 0;
    virtual void wcPredicateQueryExists() = 0;
    virtual void wcPredicateIn() = 0;
    virtual void wcPredicateNotIn() = 0;
@@ -251,6 +251,8 @@ protected:
    virtual void wcPredicateNotNullComp() = 0;
    virtual void wcPredicateLikeComp() = 0;
    virtual void wcPredicateNotLikeComp() = 0;
+   virtual void wcPredicateNcLikeComp() = 0;
+   virtual void wcPredicateNotNcLikeComp() = 0;
    virtual void wcPredicateExprOnly() = 0;
    virtual void wcPredicateNotPredicate() = 0;
    virtual void procExpr1() = 0;
@@ -319,6 +321,7 @@ protected:
    virtual void miscFloatValue() = 0;
    virtual void miscDoubleValue() = 0;
    virtual void miscDecimalValue() = 0;
+   virtual void miscDatetimeValue() = 0;
    virtual void miscSysDate() = 0;
    virtual void miscNull() = 0;
    virtual void miscTrueValue() = 0;
@@ -405,6 +408,7 @@ protected:
       RETURNTOKEN,
       REORGANIZE,
       RENAME,
+      REGMATCH,
       REFERENCES,
       RBSEG,
       RANDINT,
@@ -426,6 +430,7 @@ protected:
       NOT,
       NOOP,
       NEXTCOUNT,
+      NCLIKE,
       MODIFY,
       MOD,
       MAX,
@@ -655,6 +660,7 @@ private:
       ProcStatement,
       ProcWhileCondition,
       Condition,
+      Conjunction,
       Predicate,
       Expr,
       Term,
@@ -965,6 +971,14 @@ private:
    void loadParser291();
    void loadParser292();
    void loadParser293();
+   void loadParser294();
+   void loadParser295();
+   void loadParser296();
+   void loadParser297();
+   void loadParser298();
+   void loadParser299();
+   void loadParser300();
+   void loadParser301();
 
    class ScannerStateEntry {
 
@@ -1061,8 +1075,8 @@ class ScannerTransEntry {
       int _num;
    };
 
-   ActionEntry _actionMap[938][202];
-   int _jumpMap[938][129];
+   ActionEntry _actionMap[950][204];
+   int _jumpMap[950][130];
 
    class ProdEntry {
 
