@@ -386,11 +386,11 @@ public:
     
     void wcWhereClause();
     void wcEmptyClause();
-   
-    void wcConditionAnd();
-    void wcConditionOr();
-    void wcConditionPredicate();
 
+    void conditionAnd();
+    void conditionOr();
+    void conditionPredicate();
+    
     void wcPredicateQueryExists();
 
     void wcPredicateIn();
@@ -408,6 +408,8 @@ public:
     void wcPredicateNotNullComp();
     void wcPredicateLikeComp();
     void wcPredicateNotLikeComp();
+    void wcPredicateNcLikeComp();
+    void wcPredicateNotNcLikeComp();
     void wcPredicateExprOnly();
     void wcPredicateNotPredicate();
 
@@ -456,6 +458,7 @@ public:
     void miscFloatValue();
     void miscDoubleValue();
     void miscDecimalValue();
+    void miscDatetimeValue();
 
     void miscNull();
     void miscSysDate();
