@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // Class: CegoAdminHandler
 // 
@@ -36,23 +36,27 @@ public:
 	RETRIEVETABLESET, 
 	LOCKINFO,
 	LOCKSTAT,
+	PARAMETERINFO,
 	POOLINFO,
 	POOLENTRYLIST,
 	THREADINFO,
 	DBTHREADABORT,
 	DBTHREADINFO,
+	DBTHREADHIST,
+	QUERYHISTLAST,
+	QUERYHISTCOST,
 	ADMTHREADINFO,
 	LOGTHREADINFO,
 	DBSESSIONINFO,
 	COPYINFO,
+	SHOWQUERYCACHE,
 	LISTQUERYCACHE,
 	CLEANQUERYCACHE,
-	SETQUERYCACHEENTRY,
-	SETQUERYCACHESIZE,
+	SETQUERYCACHEPARAM,
+	SHOWTABLECACHE,
 	LISTTABLECACHE,
 	CLEANTABLECACHE,
-	SETTABLECACHEENTRY,
-	SETTABLECACHESIZE,
+	SETTABLECACHEPARAM,
 	DEFINE_TABLESET,
 	REMOVE_TABLESET,
 	RESET_TABLESET,
@@ -68,7 +72,9 @@ public:
 	START_RECOVERY,
 	STOP_RECOVERY,
 	SET_LOGMNG,
+	SET_BACKUPMNG,
 	SHOW_LOGMNG,
+	SHOW_BACKUPMNG,
 	SET_CHECKPOINT,
 	SET_TSINITFILE,
 	SET_TSSYSSIZE,
@@ -78,9 +84,11 @@ public:
 	SET_TSLOGNUM,
 	SET_TSLOGSIZE,
 	SET_TSSORTAREASIZE,
+	SET_TSLOGUSER,
 	GET_NODELIST,
 	GET_TSINFO,
 	GET_DETAILED_TSINFO,
+	GET_ARCHLOG_INFO,
 	GET_OBJECT_INFO,
 	GET_TRANSACTION_INFO,
 	GET_BUSTAT,
@@ -112,7 +120,10 @@ public:
 	ADD_ARCHLOG,
 	REMOVE_ARCHLOG,
 	BEGIN_BACKUP,
+	EXECUTE_BACKUP,
 	END_BACKUP,
+	LIST_BACKUP,
+	RESTORE,
 	RECOVER,
 	SECSWITCH,
 	VERIFY_TABLESET,
@@ -122,7 +133,9 @@ public:
 	DISABLE_TABLECACHE,
 	ENABLE_QUERYCACHE,	
 	DISABLE_QUERYCACHE,
-
+	SET_NUMQUERYLAST,
+	SET_NUMQUERYCOST,
+	
 	/* mediator requests */
 	MED_NOTIFY,
 	MED_DEFINE_TABLESET,
@@ -139,12 +152,16 @@ public:
 	MED_REMOVE_ARCHLOG,
 	MED_GET_TABLESETLIST,
 	MED_GET_DETAILEDTABLESETINFO,
+	MED_GET_ARCHLOGINFO,
 	MED_GET_OBJECTINFO,
 	MED_GET_TRANSACTIONINFO,
 	MED_GET_BUSTAT,
 	MED_RESET_BUSTAT,
 	MED_BEGIN_BACKUP,
+	MED_EXECUTE_BACKUP,
 	MED_END_BACKUP,
+	MED_LIST_BACKUP,
+	MED_RESTORE,
 	MED_RECOVER,
 	MED_SECSWITCH,
 	MED_SECRELOCATE,
@@ -155,6 +172,8 @@ public:
 	MED_DISABLE_ARCHLOG,
 	MED_ENABLE_AUTOCORRECT,	
 	MED_DISABLE_AUTOCORRECT,
+	MED_ENABLE_APPENDMODE,	
+	MED_DISABLE_APPENDMODE,
 
 	/* secondary requests */
 	SEC_MEDSWITCH,
@@ -185,25 +204,35 @@ public:
     ResultType reqLockInfo(const Chain& lockCat);
     ResultType reqLockStat();
     ResultType reqPoolInfo();
+    ResultType reqParameterInfo();
     
     ResultType reqPoolEntryList();
     ResultType reqThreadInfo();
     ResultType reqDbThreadInfo();
+    ResultType reqDbThreadHistory(int threadId);
+
+    ResultType reqQueryHistoryLast(int numLast);
+    ResultType reqQueryHistoryCost(int numLast);
+
+    ResultType reqNumQueryLast(int numLast);
+    ResultType reqNumQueryCost(int numCost);
+    
     ResultType reqAdmThreadInfo();
     ResultType reqLogThreadInfo();
     ResultType reqDbSessionInfo();
     ResultType reqCopyInfo();
 
+    ResultType reqShowQueryCache(const Chain& tableSet);
     ResultType reqListQueryCache(const Chain& tableSet);
     ResultType reqCleanQueryCache(const Chain& tableSet);
-    ResultType reqSetQueryCacheEntry(const Chain& tableSet, int numEntry);
-    ResultType reqSetQueryCacheSize(const Chain& tableSet, int cacheSize);
+    ResultType reqSetQueryCacheParam(const Chain& tableSet, int maxEntry, int maxSize, int threshold, int hashRange);
 
+    ResultType reqShowTableCache(const Chain& tableSet);
     ResultType reqListTableCache(const Chain& tableSet);
     ResultType reqCleanTableCache(const Chain& tableSet);
-    ResultType reqSetTableCacheEntry(const Chain& tableSet, int numEntry);
-    ResultType reqSetTableCacheSize(const Chain& tableSet,int cacheSize);
-    
+
+    ResultType reqSetTableCacheParam(const Chain& tableSet, const Chain& cacheFilter, int maxEntry, int maxSize, int hashRange);
+        
     ResultType reqDbThreadAbort(int threadId);
     
     ResultType reqDefineTableSet(const Chain& tableSet, 
@@ -237,6 +266,7 @@ public:
     ResultType reqGetDbInfo();
     ResultType reqGetTSInfo(const Chain& tableSet);
     ResultType reqGetDetailedTSInfo(const Chain& tableSet);
+    ResultType reqGetArchLogInfo(const Chain& tableSet);
     ResultType reqGetBUStatInfo(const Chain& tableSet);
 
     ResultType reqGetObjectInfo(const Chain& tableSet);
@@ -252,15 +282,13 @@ public:
     ResultType reqRegisterHost(const Chain& hostName, const int dataPortNo, const int adminPortNo );
 
     ResultType reqExportTableSet(const Chain& tableSet, bool isStructure, const Chain& exportFile, const Chain& expMode);
-    ResultType reqImportTableSet(const Chain& tableSet, bool isStructure, const Chain& importFile, bool doLogging, const Chain& impMode);
+    ResultType reqImportTableSet(const Chain& tableSet, bool isStructure, const Chain& importFile, const Chain& impMode);
     
     ResultType reqExportTable(const Chain& tableSet, const Chain& tableName, const Chain& exportFile, const Chain& expMode);
-    ResultType reqImportTable(const Chain& tableSet, const Chain& tableName, const Chain& importFile, bool doLogging, const Chain& impMode);
-
+    ResultType reqImportTable(const Chain& tableSet, const Chain& tableName, const Chain& importFile, const Chain& impMode);
 
     ResultType reqAddDataFile(const Chain& tableSet, const Chain& fileType, int fileId, const Chain& dataFile, int fileSize);
     
-
     ResultType reqShowUser();
     ResultType reqAddUser(const Chain& user, const Chain& password);
     ResultType reqRemoveUser(const Chain& user);
@@ -271,7 +299,6 @@ public:
     ResultType reqCreateRole(const Chain& role);
     ResultType reqDropRole(const Chain& role);
 
-
     ResultType reqSetPermission(const Chain& role, const Chain& permid, const Chain& tableSet, const Chain& filter, const Chain& perm);
     ResultType reqRemovePermission(const Chain& role, const Chain& permid);
 
@@ -284,14 +311,19 @@ public:
     ResultType reqRemoveArchLog(const Chain& tableSet, const Chain& archId);
 
     ResultType reqBeginBackup(const Chain& tableSet);
+    ResultType reqExecuteBackup(const Chain& tableSet);
     ResultType reqEndBackup(const Chain& tableSet, bool keepTicket);
-    ResultType reqRecover(const Chain& tableSet, int pit);
+    ResultType reqListBackup(const Chain& tableSet, const Chain& pit);
+    ResultType reqRestore(const Chain& tableSet, const Chain& backupId);
+    ResultType reqRecover(const Chain& tableSet, const Chain& pit);
     ResultType reqSecSwitch(const Chain& tableSet, const Chain& secondary);
 
     ResultType reqDumpObject(const Chain& tableSet, CegoObject::ObjectType type, const Chain& objName);
 
     ResultType reqSetLogMng(const Chain& progName, int timeout);
+    ResultType reqSetBackupMng(const Chain& progName);
     ResultType reqShowLogMng();
+    ResultType reqShowBackupMng();
 
     ResultType reqSetCheckpoint(const Chain& tableSet, int interval);
     ResultType reqSetTSInitFile(const Chain& tableSet, const Chain& initFile);
@@ -304,13 +336,13 @@ public:
     ResultType reqSetTSLogNum(const Chain& tableSet, int logNum);
     ResultType reqSetTSLogSize(const Chain& tableSet, int logSize);
     ResultType reqSetTSSortAreaSize(const Chain& tableSet, unsigned long long sortAreaSize);
+    ResultType reqSetTSLogUser(const Chain& tableSet, const Chain& logUser);
 
     ResultType reqEnableTableCache(const Chain& tableSet);
     ResultType reqDisableTableCache(const Chain& tableSet);
     
     ResultType reqEnableQueryCache(const Chain& tableSet);
     ResultType reqDisableQueryCache(const Chain& tableSet);
-
     
     ResultType sendReq(const Chain& reqType, Element* pRoot);
 
@@ -318,7 +350,7 @@ public:
     ResultType medNotifyMediator(const Chain& hostName, const Chain& status, 
 				 const ListT<Chain>& tsList, const ListT<Chain>& runList, const ListT<Chain>& syncList);
 
-    ResultType medStartTableSet(const Chain& tableSet, bool cleanIt, bool forceload);
+    ResultType medStartTableSet(const Chain& tableSet, bool cleanIt, bool forceload, bool cpDump, bool noInit);
     ResultType medStopTableSet(const Chain& tableSet);
 
     ResultType medDefineTableSet(const Chain& tableSet, 
@@ -342,18 +374,22 @@ public:
     ResultType medGetTableSetList(bool getUsage = false);
     ResultType medGetDetailedTableSetInfo(const Chain& tableSet);
     ResultType medGetObjectInfo(const Chain& tableSet);
+    ResultType medGetArchLogInfo(const Chain& tableSet);
     ResultType medGetTransactionInfo(const Chain& tableSet);
     ResultType medGetBUStatInfo(const Chain& tableSet);
     ResultType medResetBUStat(const Chain& tableSet);
 
     ResultType medBeginBackup(const Chain& tableSet, const Chain& msg);
+    ResultType medExecuteBackup(const Chain& tableSet, const Chain& msg);
     ResultType medEndBackup(const Chain& tableSet, const Chain& msg, bool keepTicket);
+    ResultType medListBackup(const Chain& tableSet, const Chain& pit);
+    ResultType medRestore(const Chain& tableSet, const Chain& backupId);   
     ResultType medRecover(const Chain& tableSet, const Chain& pit);
+
     ResultType medSecSwitch(const Chain& tableSet);
 
     ResultType medSecRelocate(const Chain& tableSet, const Chain& hostName);
 
-
     ResultType medCheckTableSet(const Chain& tableSet);
     ResultType medVerifyTableSet(const Chain& tableSet);
     ResultType medCorrectTableSet(const Chain& tableSet);
@@ -361,6 +397,9 @@ public:
     ResultType medEnableArchLog(const Chain& tableSet);
     ResultType medDisableArchLog(const Chain& tableSet);
 
+    ResultType medEnableAppendMode(const Chain& tableSet);
+    ResultType medDisableAppendMode(const Chain& tableSet);
+
     ResultType medEnableAutoCorrect(const Chain& tableSet);
     ResultType medDisableAutoCorrect(const Chain& tableSet);
 
@@ -373,8 +412,9 @@ public:
     void sendResponse(const Chain& msg, Element* pSubElement = 0);
     void sendError(const Chain& msg);
     void sendData(const Chain& data);
-
+    
     ResultType nextInfo();
+    ResultType abort();
     bool syncWithInfo(const Chain& hostRole, const Chain& hostName, const Chain& msg, Element* pInfo = 0);
 
     ////////////////////////////
@@ -388,12 +428,14 @@ public:
     void getBUMsg(Chain& msg);
     void getLockInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getLockStat(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
-    void getPoolInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info1, 
-		     ListT<ListT< CegoFieldValue > > & info2);
+    void getPoolInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
+
+    void getParameterInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
+    
     void getPoolEntryList(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getThreadInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getDbThreadInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
-    void getDbThreadLastQuery(int threadId, CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
+    void getDbThreadHistory(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
 
     void getAdmThreadInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
     void getLogThreadInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
@@ -408,6 +450,7 @@ public:
     Element* getTableSetCheckInfo();
 
     Element* getTSInfo();
+    Element* getArchInfo();
     Element* getObjectInfo();
     Element* getTAInfo();
 
@@ -420,17 +463,19 @@ public:
     void getObjectInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getTransactionInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getBUStatInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
+    void getBackupInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getNodeInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getUserInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getTableSetVerificationInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getTableSetCorrectionInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
 
-
     void getRoleList( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getRoleInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
 
     void getLogMngInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
+    void getBackupMngInfo( CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
 
+    void getBackupId(Chain& backuId);
     void getTableSet(Chain& tableSet);
     void getTableSetRoot(Chain& tsRoot);
     void getTableSetTSTicket(Chain& tsTicket);
@@ -445,7 +490,8 @@ public:
     void getEscapeCmd(Chain& escCmd);
     void getTSInitFile(Chain& initFile);
     void getTSRootPath(Chain& tsRoot);
-
+    void getTSLogUser(Chain& logUser);
+	
     void getTSTicket(Chain& tsRoot);
     void getTSLogNum(int& logNum);
     void getTSLogSize(int& logSize);
@@ -464,6 +510,8 @@ public:
     void getLogManager(Chain& progName);
     void getLogManagerTimeout(int& timeout);
 
+    void getBackupManager(Chain& progName);
+    
     void getLockCat(Chain& lockCat);
     void getPrimary(Chain& primary);
     void getSecondary(Chain& secondary);
@@ -489,20 +537,30 @@ public:
     void getSyncState(Chain& syncState);
     void getPit(Chain& pit);
     void getIsStructure(bool& isStructure);
-    void getDoLogging(bool& doLogging);
     void getTableSyncStateList(ListT<Chain>& tsList, ListT<Chain>& runList, ListT<Chain>& syncList);
     void getKeepTicket(bool& keepTicket);
     void getMode(Chain& mode);
     bool getCleanup();
     bool getForceload();
-
+    bool getCPDump();
+    bool getNoInit();
+    
     void getObjName(Chain& objName);
     void getObjType(CegoObject::ObjectType& type);
 
-    void getCacheEntry(int& numEntry);
-    void getCacheSize(int& cacheSize);
+    void getTableCacheFilter(Chain& cacheFilter);
+    void getTableCacheMaxSize(int& maxSize);
+    void getTableCacheMaxEntry(int& maxEntry);
+    void getTableCacheHashRange(int& hashRange);
     
+    void getQueryCacheMaxEntry(int& maxEntry);
+    void getQueryCacheMaxSize(int& maxSize);
+    void getQueryCacheThreshold(int& threshold);
+    void getQueryCacheHashRange(int& hashRange);
 
+    void getNumLast(int& numLast);
+    void getNumCost(int& numCost);
+    
 private:
 
     NetHandler *_pN;
@@ -514,11 +572,4 @@ private:
 
     unsigned long _modId;
 };
-
-
-
 #endif
-
-
-
-
