@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,7 +20,6 @@
 
 // CEGO INCLUDES
 #include "CegoBTreeObject.h"
-#include "CegoXMLHelper.h"
 #include "CegoTypeConverter.h"
 #include "CegoXMLdef.h"
 
@@ -32,28 +31,20 @@ CegoBTreeObject::CegoBTreeObject()
 {
 }
 
-CegoBTreeObject::CegoBTreeObject(Element *pTO)
-{
-    putElement(pTO);
-}
-
 CegoBTreeObject::CegoBTreeObject(const CegoBTreeObject& oe) : CegoContentObject(oe)
 {
-    _fileId = oe._fileId;
     _pageId = oe._pageId;
     _relevance = oe._relevance;
 }
 
 CegoBTreeObject::CegoBTreeObject(int tabSetId, CegoObject::ObjectType type, const Chain& objName ) : CegoContentObject(tabSetId, type, objName)
 {
-    _fileId = 0;
     _pageId = 0;
     _relevance = 0;
 }
 
 CegoBTreeObject::CegoBTreeObject(int tabSetId, CegoObject::ObjectType type,  const Chain& objName, const ListT<CegoField>& schema, const Chain& tabName) : CegoContentObject(tabSetId, type, objName, tabName, schema)
 {
-    _fileId = 0;
     _pageId = 0;
     _relevance=0;
 }
@@ -67,22 +58,12 @@ void CegoBTreeObject::setSchema(const ListT<CegoField>& schema)
     _schema = schema;
 }
 
-void CegoBTreeObject::setDataFileId(int fileId)
-{
-    _fileId = fileId;
-}
-
-int CegoBTreeObject::getDataFileId() const
-{
-    return _fileId;
-}
-
-void CegoBTreeObject::setDataPageId(int pageId)
+void CegoBTreeObject::setDataPageId(PageIdType pageId)
 {
     _pageId = pageId;
 }
 
-int CegoBTreeObject::getDataPageId() const
+PageIdType CegoBTreeObject::getDataPageId() const
 {
     return _pageId;
 }
@@ -104,17 +85,15 @@ int CegoBTreeObject::getRelevance() const
 
 int CegoBTreeObject::getEntrySize() const
 {
-    
     int entrySize = CegoContentObject::getBaseContentSize();
 
-    entrySize = entrySize + sizeof(int); // fileId
-    entrySize = entrySize + sizeof(int); // pageId
+    entrySize = entrySize + sizeof(PageIdType); // pageId
     entrySize = entrySize + sizeof(int); // relevance
         
     return entrySize;
 }
 
-void CegoBTreeObject::encode(char *buf)
+void CegoBTreeObject::encode(char *buf) const
 {
     char* bufPtr = buf;
 
@@ -123,15 +102,11 @@ void CegoBTreeObject::encode(char *buf)
     CegoContentObject::encodeBaseContent(bufPtr, entrySize);
     bufPtr += CegoContentObject::getBaseContentSize();
 
-    memcpy(bufPtr, &_fileId, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-    
-    memcpy(bufPtr, &_pageId, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
+    memcpy(bufPtr, &_pageId, sizeof(PageIdType));
+    bufPtr = bufPtr + sizeof(PageIdType);
 
     memcpy(bufPtr, &_relevance, sizeof(int));
     bufPtr = bufPtr + sizeof(int);
-
 }
 
 void CegoBTreeObject::decode(char *buf)
@@ -143,31 +118,23 @@ void CegoBTreeObject::decode(char *buf)
     CegoContentObject::decodeBaseContent(bufPtr, size);
     bufPtr += CegoContentObject::getBaseContentSize();
     
-    memcpy(&_fileId, bufPtr, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
-
-    memcpy(&_pageId, bufPtr, sizeof(int));
-    bufPtr = bufPtr + sizeof(int);
+    memcpy(&_pageId, bufPtr, sizeof(PageIdType));
+    bufPtr = bufPtr + sizeof(PageIdType);
 
     memcpy(&_relevance, bufPtr, sizeof(int));
     bufPtr = bufPtr + sizeof(int);
-        
 }
 
 bool CegoBTreeObject::isValid()
 {
-    return  _fileId != 0 || _pageId != 0;
+    return _pageId != 0;
 }
 
 CegoBTreeObject& CegoBTreeObject::operator = ( const CegoBTreeObject& oe) 
 {
-
     CegoContentObject::operator=(oe);
-
-    _fileId = oe._fileId;
     _pageId = oe._pageId;
     _relevance = oe._relevance;
-
     return (*this);
 }
 
@@ -176,118 +143,21 @@ bool CegoBTreeObject::operator == ( const CegoBTreeObject& oe)
     return CegoContentObject::operator==(oe);
 }
 
-Chain CegoBTreeObject::getId() const
+Chain CegoBTreeObject::getId(CegoProcBlock *pBlock) const
 {
     return getTabName() + Chain("@") + getTableSet() + Chain("@") + getName();
 }
 
-Chain CegoBTreeObject::toChain() const
+Chain CegoBTreeObject::toChain(int defTabSetId) const
 {
-    return getTabName() + Chain("@") + getTableSet() + Chain(" ") + getName();
-}
-
-Element* CegoBTreeObject::getElement() const
-{
-    Element* pRoot = new Element(XML_OBJ_ELEMENT);
-
-    pRoot->setAttribute(XML_TSID_ATTR, Chain(getTabSetId()));
-    switch ( getType() )
-    {
-    case CegoObject::BTREE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_BTREEOBJ_VALUE);	
-	break;
-    case CegoObject::PBTREE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_PBTREEOBJ_VALUE);	
-	break;
-    case CegoObject::UBTREE:
-	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_UBTREEOBJ_VALUE);	
-	break;
-    default:
-	break;
-    }
-
-    pRoot->setAttribute(XML_OBJNAME_ATTR, getName());
-
-    CegoField *pF = _schema.First();
-    while ( pF ) 
-    {
-	Element *pColElement = new Element(XML_SCHEMA_ELEMENT);
-
-	CegoXMLHelper xh;
-	xh.setColInfo(pColElement, pF);
-
-	pRoot->addContent(pColElement);
-	pF = _schema.Next();
-    }
-
-    return pRoot;
-}
-
-void CegoBTreeObject::putElement(Element* pElement)
-{
-    Element *pRoot = pElement;
-    
-    if ( pRoot )
-    {
-	
-	Chain objName = pRoot->getAttributeValue(XML_OBJNAME_ATTR); 
-	int tabSetId = pRoot->getAttributeValue(XML_TSID_ATTR).asInteger();
-	
-	setName(objName);
-	setTabName(objName);
-	setTabSetId(tabSetId);
-
-	Chain objType = pRoot->getAttributeValue(XML_OBJTYPE_ATTR);
-	if ( objType == Chain(XML_BTREEOBJ_VALUE))
-	    setType(CegoObject::BTREE);
-	else if ( objType == Chain(XML_PBTREEOBJ_VALUE))
-	    setType(CegoObject::PBTREE);
-	else if ( objType == Chain(XML_UBTREEOBJ_VALUE))
-	    setType(CegoObject::UBTREE);
-
-	ListT<Element*> colList = pRoot->getChildren(XML_SCHEMA_ELEMENT);
-	
-	ListT<CegoField> fl;
-	Element **pCol = colList.First();
-	while ( pCol ) 
-	{
-	    Chain colName = (*pCol)->getAttributeValue(XML_COLNAME_ATTR);
-	    Chain colType = (*pCol)->getAttributeValue(XML_COLTYPE_ATTR);
-	    Chain colSize = (*pCol)->getAttributeValue(XML_COLSIZE_ATTR);
-
-	    Chain colNullable = (*pCol)->getAttributeValue(XML_COLNULLABLE_ATTR);
-	    Chain colDefValue = (*pCol)->getAttributeValue(XML_COLDEFVALUE_ATTR);
-
-
-	    bool isNullable;
-	    if ( colNullable == Chain(XML_TRUE_VALUE) ) 
-		isNullable = true;
-	    else
-		isNullable = false;
-
-	    
-	    CegoTypeConverter tc;
-	    CegoDataType dataType = tc.getTypeId(colType);
-
-	    CegoFieldValue defValue;
-	    if ( colDefValue != Chain("") )
-	    {
-		defValue = CegoFieldValue(dataType, colDefValue);
-	    }
-
-	    CegoField f(CegoField(objName, objName, colName, dataType, colSize.asInteger(), defValue, isNullable));		
-	    fl.Insert(f);
-
-	    pCol = colList.Next();
-	}
-	
-	setSchema(fl);
-    }
+    if  ( defTabSetId == getTabSetId() )
+	return getTabName() + Chain(" ") + getName();
+    else
+	return getTabName() + Chain("@") + getTableSet() + Chain(" ") + getName();
 }
 
 Chain CegoBTreeObject::getFormatted() const
 {
-
     Chain s;
 	
     int maxAttrLen = 12;
@@ -319,7 +189,20 @@ Chain CegoBTreeObject::getFormatted() const
     case CegoObject::PBTREE:
 	s += Chain("primary btree   ");
 	break;
-    default:
+    case CegoObject::SYSTEM:
+    case CegoObject::TABLE:
+    case CegoObject::VIEW:
+    case CegoObject::RBSEG:
+    case CegoObject::FKEY:
+    case CegoObject::PROCEDURE:
+    case CegoObject::CHECK:
+    case CegoObject::JOIN:
+    case CegoObject::PAVLTREE:
+    case CegoObject::UAVLTREE:
+    case CegoObject::AVLTREE:
+    case CegoObject::TRIGGER:
+    case CegoObject::ALIAS:
+    case CegoObject::UNDEFINED:
 	break;
     }
 
@@ -332,11 +215,9 @@ Chain CegoBTreeObject::getFormatted() const
     pF = _schema.First();
     while (pF)
     {
-	
 	int num = maxAttrLen - pF->getAttrName().length() ;
 	s += Chain("| ") + pF->getAttrName() + fill(" ", num) + Chain(" |");
-	    
-	    
+	    	    
 	switch (pF->getType())
 	{
 	case INT_TYPE:
@@ -387,12 +268,6 @@ Chain CegoBTreeObject::getFormatted() const
 	    s += Chain("  decimal[") + l + Chain("]") + fill(" ", 5 - l.length()) + Chain("|");
 	    break;
 	}
-	case FIXED_TYPE:
-	{
-	    Chain l(pF->getLength());
-	    s += Chain("  fixed[") + l + Chain("]") + fill(" ", 7 - l.length()) + Chain("|");
-	    break;
-	}
 	case FLOAT_TYPE:
 	{
 	    s += Chain(" float         |");
@@ -408,7 +283,13 @@ Chain CegoBTreeObject::getFormatted() const
 	    s += Chain(" blob          |");
 	    break;
 	}
-	default:
+	case CLOB_TYPE:
+	{
+	    s += Chain(" clob          |");
+	    break;
+	}
+	case NULL_TYPE:
+	case PAGEID_TYPE:
 	    break;
 	}
 	
@@ -428,13 +309,11 @@ Chain CegoBTreeObject::getFormatted() const
     s += Chain("+") + fill("-", maxAttrLen + 1) + Chain("+---------------+") + fill("-", maxDefLen + 1) + Chain("+----------+\n");
 
     return s;
-
 }
 
 Chain CegoBTreeObject::fill(const Chain& s, int num) const
-{
-    
-    Chain fs = Chain("");
+{    
+    Chain fs = Chain();
     while (num > 0)
     {
 	fs = fs + s;
