@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -35,10 +35,14 @@ public:
     enum Comparison { LESS, EQUAL, MORE };
     
     CegoBTreeValue();
-    CegoBTreeValue(char *p);
+    CegoBTreeValue(char *p, int len);
     ~CegoBTreeValue();
 
+    static bool isNullTerminated(CegoDataType type);
     static int getKeyLen(const ListT<CegoField>& schema);
+    static int getKeyPos(const Chain& attrName, const ListT<CegoField>& schema);
+
+    bool isNull(const ListT<CegoField>& schema) const;
     bool hasNull(const ListT<CegoField>& schema) const;
     
     void valueFromSchema(ListT<CegoField>* pSchema);
@@ -46,7 +50,7 @@ public:
 
     ListT<CegoFieldValue> valueToFVL(const ListT<CegoField>& schema) const;
 
-    void setPtr(char* p);
+    void setPtr(char* p, int len);
     char* getPtr() const;
     
     CegoBTreeValue& operator = (const CegoBTreeValue& iv);
@@ -54,14 +58,18 @@ public:
     Comparison comp(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const;
 
     bool isHigher(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const;
+    bool isEqualHigher(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const;
     bool isEqual(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const;
-    bool operator > ( const CegoBTreeValue& iv) const;
+    // bool operator > ( const CegoBTreeValue& iv) const;
 
     Chain toChain(ListT<CegoField>* pSchema) const;    
-
+	
 private:
+
+    static int getReservedLength(CegoField* pF);
     
     char *_pI;
-    char _idxArray[TABMNG_MAXINDEXVALUE];
+    int _len;
+    char* _idxArray; 
 };
 #endif
