@@ -54,6 +54,7 @@
 #define DEFAULTPROTOCOL "serial"
 #define DEFAULTMAXLINE 10000
 #define DEFAULTMAXRESULT 100000
+#define DEFAULTPACKSIZE 500
 
 #define USAGE "Usage: cgclt --user=<user>/<password>\n\
           [ --server=<host> ]\n\
@@ -69,12 +70,14 @@
           [ --pager ]\n\
           [ --maxline=<maxline> ]\n\
           [ --maxresult=<maxresult> ]\n\
+          [ --pack=<packsize> ]\n\
           [ --ignore ]\n\
           [ --profile=<profile id> ]\n\
           [ --debug ] [ --version  ] [ --help ]"
 
 #define DEFAULTRAWSEP ","
 #define HISTFILE ".cgclt_history"
+#define PGHISTFILE ".cgclt_pghist"
 #define PROMPT "CGCLT > "
 #define PROMPT2 "> "
 #define CGPROFILE ".cgprofile"
@@ -83,11 +86,12 @@
 #define PIPE_TIMEOUT 30 /* checking input pipe data timeout in ms */
 #define HOMEVAR "HOME"
 
-enum RunMode { INTERACTIVE, BATCH, DUMP };
+enum RunMode { INTERACTIVE, PAGER, BATCH, DUMP };
 
 extern char __lfcVersionString[];
 extern char __lfcxmlVersionString[];
 extern Chain __dateTimeFormat;
+extern int __dateLength;
 extern char __currencySymbol;
 extern char __decimalPoint;
 
@@ -102,7 +106,8 @@ public:
 		 const Chain& user, 
 		 const Chain& password,		 
 		 unsigned long long tid, 
-		 CegoModule *pModule);
+		 CegoModule *pModule,
+		 bool isQuiet);
     ~AbortHandler();
     void sigCatch(int sig);
 
@@ -118,6 +123,7 @@ private:
     Chain _user;
     Chain _password;
     Chain _tableSet;
+    bool _isQuiet;
 };
 
 class ProfileEntry
@@ -158,9 +164,139 @@ private:
     Chain _prompt;
 };
 
+
+class CegoPager : public Pager
+{
+    
+public:
+    
+    CegoPager(const Chain& title, CegoNet* pNet, unsigned long long maxResult) : Pager(title) {
+	_pNet = pNet;
+	_maxResult = maxResult;
+    }
+    
+    ~CegoPager() {
+    }
+
+    void setAbortHandler(AbortHandler *pAH) {
+	_pAH = pAH;
+    }
+    
+    bool execute(const Chain& cmd, Chain& msg) {
+	
+	_pagerSchema.Empty();
+	_outData.Empty();
+	_hasData=false;
+	
+	if ( _pAH )
+	    _pAH->setQueryAbort(true);
+
+	try
+	{
+	    // cout << "Query= <<<" << cmd << ">>>" << endl;
+	    // doQuery return number of affected, which we do not use here
+	    _pNet->doQuery(cmd);
+	}
+	catch ( Exception e )
+	{
+	    e.pop(msg);
+	    
+	    if ( _pAH )
+		_pAH->setQueryAbort(false);
+	    
+	    return false;
+	}
+
+	if ( _pNet->isFetchable() )
+	{
+	    try
+	    {					
+		ListT<CegoField> schema;
+		_pNet->getSchema(schema);
+		
+		ListT<CegoFieldValue> fvl;
+		
+		unsigned long long count=0;
+						
+		while ( _pNet->fetchData(schema, fvl)  )
+		{
+		    count++;
+		    
+		    if ( count > _maxResult )
+		    {
+			_pNet->abortQuery();
+			msg=Chain("Result is too large");
+			return false;
+		    }
+		    
+		    
+		    CegoFieldValue *pFV = fvl.First();
+		    ListT<Chain> outRow;
+		    while ( pFV )
+		    {
+			outRow.Insert(pFV->valAsChain());
+			pFV = fvl.Next();		    
+		    }
+		    _outData.Insert(outRow);
+		}
+		fvl.Empty();	    
+				
+		CegoField *pF = schema.First();
+		while ( pF )
+		{
+		    Chain attrDesc = pF->getAttrName() + Chain(":auto"); //  + Chain(CegoQueryHelper::maxFieldSize(pF));
+		    _pagerSchema.Insert(attrDesc);		
+		    pF = schema.Next();		
+		}
+
+		_hasData=true;
+		msg=Chain(count) + Chain(" tuples");
+	    }
+	    catch ( Exception e )
+	    {
+		_pNet->abortQuery();
+		e.pop(msg);
+	    }
+	}
+	else
+	{	      
+	    msg = _pNet->getMsg();
+	}
+	return true;
+    }
+
+    bool hasData() {
+	return _hasData;
+    }
+    
+    ListT<Chain> getSchema() {
+
+	return _pagerSchema;
+    }
+    
+    ListT< ListT<Chain> > getData() {
+
+	return _outData;
+	
+    }
+
+private:
+
+    CegoNet *_pNet;
+    AbortHandler *_pAH;
+
+    unsigned long long _maxResult;
+
+    bool _hasData;
+    ListT<Chain> _pagerSchema;
+    ListT< ListT<Chain> > _outData;
+            
+};
+
+
 int execute();
 void treatEscape(Chain& cmd);
-bool dispatchCmd(bool outputBuffered, CegoNet* pCegoNet, AbortHandler* pAH, Chain cmd, bool rawMode, bool noHead, int& exitCode, Chain& msg);
+bool dispatchCmd(CegoNet* pCegoNet, AbortHandler* pAH, Chain cmd, bool rawMode, bool noHead, int& exitCode, Chain& msg);
 void loadProfile(const Chain& rcFileName, ListT<ProfileEntry>& pfeList);
 void dumpProlog(File *pDF);
 void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure);
@@ -185,9 +321,9 @@ Chain protocol;
 
 unsigned long long maxLine;
 unsigned long long maxResult;
+unsigned long long packSize;
 bool rawMode = false;
 bool noHead = false;
-bool isBuffered = false;
 Chain rawSep;
 bool ignoreError = false;
 bool isStructure = false;
@@ -201,6 +337,7 @@ CegoModule *pModule = 0;
 unsigned long modId;
 CegoDbHandler::ProtocolType protType;
 Chain histPath;
+Chain pgHistPath;
 
 int main(int argc, char **argv)
 {    
@@ -216,7 +353,7 @@ int main(int argc, char **argv)
     longOpt.addOpt("structure");
     longOpt.addOpt("maxline", Chain(DEFAULTMAXLINE));
     longOpt.addOpt("maxresult", Chain(DEFAULTMAXRESULT));
-    
+    longOpt.addOpt("pack", Chain(DEFAULTPACKSIZE));
     longOpt.addOpt("debug");
     longOpt.addOpt("raw", Chain(DEFAULTRAWSEP));
     longOpt.addOpt("nohead");
@@ -244,9 +381,20 @@ int main(int argc, char **argv)
     
 #ifdef HAVE_MINGW
     histPath = homeDir + "\\" + Chain(HISTFILE);
+    pgHistPath = homeDir + "\\" + Chain(PGHISTFILE);
 #else
     histPath = homeDir + "/" + Chain(HISTFILE);
+    pgHistPath = homeDir + "/" + Chain(PGHISTFILE);
 #endif
+
+    if ( longOpt.isSet( Chain("version") ) )
+    {
+	cout << CEGO_PRODUCT << " SQL Client (" << sizeof(void*) * 8 << " bit), Version " << CEGO_VERSION 
+	     << " [ lfc: " << __lfcVersionString  
+	     << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
+	cout << CEGO_COPYRIGHT << endl;
+	exit(0);
+    }
     
     if ( longOpt.isSet("debug") )
 	debug=true;
@@ -261,7 +409,7 @@ int main(int argc, char **argv)
 	noHead=true;
 
     if ( longOpt.isSet("pager") )
-	isBuffered=true;
+	runMode = PAGER;
     
     if ( longOpt.isSet("ignore") )
 	ignoreError=true;
@@ -275,14 +423,6 @@ int main(int argc, char **argv)
 	exit(0);
     }
 
-    if ( longOpt.isSet( Chain("version") ) )
-    {
-	cout << CEGO_PRODUCT << " SQL Client (" << sizeof(void*) * 8 << " bit), Version " << CEGO_VERSION 
-	     << " [ lfc: " << __lfcVersionString  
-	     << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
-	cout << CEGO_COPYRIGHT << endl;
-	exit(0);
-    }
 
     ListT<ProfileEntry> pfeList;
     
@@ -400,6 +540,11 @@ int main(int argc, char **argv)
     maxLine = longOpt.getOptValue("maxline").asUnsignedLongLong();
     maxResult = longOpt.getOptValue("maxresult").asUnsignedLongLong();
 
+    if ( rawMode )
+	packSize=0;
+    else
+	packSize = longOpt.getOptValue("pack").asUnsignedLongLong();
+    
     if ( logFile.length() > 0 )
 	pModule = new CegoModule(logFile, Chain());
     else
@@ -479,7 +624,11 @@ int execute()
 	    }
 
 	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
-		    
+
+	    Datetime d(PROOFDATE_VALUE, PROOFDATE_FORMAT);
+	    __dateLength = d.asChain(__dateTimeFormat).length();
+
+	    
 	    Chain cmd;
 	    Chain line;
 	    bool disableDelimiter=false;
@@ -520,7 +669,7 @@ int execute()
 				t.start();
 				
 				Chain msg;
-				goOn = dispatchCmd(false, pCegoNet, 0, cmd, rawMode, noHead, exitCode, msg);
+				goOn = dispatchCmd(pCegoNet, 0, cmd, rawMode, noHead, exitCode, msg);
 				
 				t.stop();
 				
@@ -612,10 +761,16 @@ int execute()
 	try
 	{	    
 	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
+
 	    
 	    unsigned long long tid = pCegoNet->getTid();
 	    
-	    AbortHandler *pAH = new AbortHandler(serverName, portNo, tableSet, user, password, tid, pModule);
+	    AbortHandler *pAH = new AbortHandler(serverName, portNo, tableSet, user, password, tid, pModule, false);
+
+	    // explicit init, since there is no other use of sighandler
+	    pAH->init();
+
+
 	    
 #ifdef DEBUG	
 	    pModule->log(modId, Logger::DEBUG, Chain("Got session ..."));
@@ -624,8 +779,11 @@ int execute()
 	    Chain dbName = pCegoNet->getDbName();
 	    Chain dbVersion = pCegoNet->getDbVersion();
 
+	    Datetime d(PROOFDATE_VALUE, PROOFDATE_FORMAT);
+	    __dateLength = d.asChain(__dateTimeFormat).length();
+
 	    cout << "Connected to " << dbName << " ( Version " << dbVersion << " )" << endl;
-	    
+
 	    read_history((char*)histPath);
  
 	    bool goOn = true;
@@ -649,12 +807,17 @@ int execute()
 			if (pC)
 			    free(pC);	    
 			
-			cs = cs.cutTrailing(Chain(" \t"));
+			cs = cs.cutTrailing(Chain(" \t\r\n"));
 			
 			if ( cs.length() > 1)
 			{		
 			    if ( cs[cs.length()-2] == ';')
+			    {
+				// cut multiple semicolon
+				cs = cs.cutTrailing(Chain(";"));
+				cs = cs + Chain(";");				
 				stmtComplete = true;
+			    }
 			}
 			
 			if ( ! stmtComplete )
@@ -686,7 +849,7 @@ int execute()
 			t.start();
 			
 			Chain msg;
-			goOn = dispatchCmd(isBuffered, pCegoNet, pAH, cs, rawMode, noHead, exitCode, msg);
+			goOn = dispatchCmd(pCegoNet, pAH, cs, rawMode, noHead, exitCode, msg);
 			
 			t.stop();
 			
@@ -726,7 +889,56 @@ int execute()
 	    return 1;
 	}
 	break;
-    }    
+    }
+    case PAGER:
+    {
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(), logMode );	
+	
+	try
+	{	    
+	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
+	    
+	    unsigned long long tid = pCegoNet->getTid();
+
+
+	    CegoPager pager(Chain("Pager"), pCegoNet, maxResult);
+
+	    pager.readHistory(pgHistPath);
+	    
+	    // The pager class disables Cntrl-C in constructor,
+	    // so we have to instantiate and set abort handler with explicit method
+	    AbortHandler *pAH = new AbortHandler(serverName, portNo, tableSet, user, password, tid, pModule, true);
+
+	    // no signal init of abort hander, since this is alread done by pager
+	    // another SigHandler::init would reset this signal configuration
+
+	    pager.setAbortHandler(pAH);
+	    
+#ifdef DEBUG	
+	    pModule->log(modId, Logger::DEBUG, Chain("Got session ..."));
+#endif
+
+	    Chain dbName = pCegoNet->getDbName();
+	    Chain dbVersion = pCegoNet->getDbVersion();
+
+	    // cout << "Connected to " << dbName << " ( Version " << dbVersion << " )" << endl;
+	    
+	    pager.manage();
+
+	    pager.writeHistory(pgHistPath);	    
+
+	    delete pCegoNet;
+	    delete pAH;
+	}
+	catch ( Exception e)
+	{
+	    delete pCegoNet;
+	    Chain msg;
+	    e.pop(msg);
+	    cerr << msg << endl;
+	    return 1;
+	}    
+    }
     }
     return exitCode;
 }
@@ -744,7 +956,7 @@ void treatEscape(Chain& cmd)
 	cmd = bval;
 }
 
-bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chain cmd, bool rawMode, bool noHead, int& exitCode, Chain& msg)
+bool dispatchCmd(CegoNet *pCegoNet, AbortHandler *pAH, Chain cmd, bool rawMode, bool noHead, int& exitCode, Chain& msg)
 {
     exitCode = 0;
 
@@ -781,17 +993,15 @@ bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chai
 
 	Chain format;
 	pCegoNet->getFormat(format);
-	
-	CegoOutput output(schema, format);
 
-	output.setRawMode(rawMode);
-	if ( rawMode )
-	    output.setRawSeparator(rawSep);
-
-	ListT< ListT<Chain> > outData;
-
-	if ( outputBuffered == false )
+	if ( packSize == 0 )
 	{
+	    CegoOutput output(schema, format);
+	    
+	    output.setRawMode(rawMode);
+	    if ( rawMode )
+		output.setRawSeparator(rawSep);
+	    
 	    if ( noHead == true )
 	    {
 		if ( rawMode == false )
@@ -803,74 +1013,128 @@ bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chai
 	    {
 		output.headOut();
 	    }
-	}
-
-	ListT<CegoFieldValue> fvl;
-	
-	unsigned long long count=0;
-
-	try
-	{
-	    while ( pCegoNet->fetchData(schema, fvl)  )
+	    
+	    ListT<CegoFieldValue> fvl;
+	    
+	    unsigned long long count=0;
+	    
+	    try
 	    {
-		count++;
-		
-		if ( count > maxResult )
+		while ( pCegoNet->fetchData(schema, fvl) )
 		{
-		    pCegoNet->abortQuery();
-		    exitCode = 1;
-		    msg=Chain("Result is too large");
-		    return false;
+		    count++;
+		    
+		    if ( count > maxResult )
+		    {
+			pCegoNet->abortQuery();
+			exitCode = 1;
+			msg=Chain("Result is too large");
+			return false;
+		    }
+		    
+		    output.rowOut(fvl);
+		    		
+		    fvl.Empty();	    
 		}
 		
-		if ( outputBuffered )
+		output.tailOut();
+		
+		msg=Chain(count) + Chain(" tuples");
+	    }
+	    catch ( Exception e )
+	    {
+		pCegoNet->abortQuery();
+		e.pop(msg);
+		exitCode=1;
+	    }
+	}
+	else
+	{
+	   	    
+	    try
+	    {
+
+		unsigned long long count=0;
+		
+		bool hasData=true; 
+		while ( hasData )
 		{
-		    CegoFieldValue *pFV = fvl.First();
-		    ListT<Chain> outRow;
+		    // collect data
+		    int i=0;
+		    ListT<CegoFieldValue > pfvl;		    
+		    ListT<CegoFieldValue> fvl;
+
+		    while ( hasData && i < packSize )
+		    {
+			hasData = pCegoNet->fetchData(schema, fvl);
+			if ( hasData )
+			{
+			    count++;
+			    CegoFieldValue *pFV = fvl.First();
+			    while ( pFV )
+			    {
+				pfvl.Insert(pFV->getLocalCopy());
+				pFV = fvl.Next();
+			    }
+			    i++;
+			}
+		    }
+
+		    if ( format != Chain("m") )
+		    {
+			// pack schema
+			bool firstRow=true;
+			CegoFieldValue *pFV = pfvl.First();
+			while ( pFV )
+			{
+			    CegoField *pF = schema.First();
+			    while ( pF && pFV )
+			    {			       
+				if ( firstRow || pF->getLength() < pFV->getLength() )
+				{
+				    pF->setLength(pFV->getLength());
+				}
+				pF = schema.Next();
+				pFV = pfvl.Next();
+			    }
+			    firstRow=false;
+			}
+		    }
+		    
+		    CegoOutput output(schema, format);
+
+		    bool headOut = true;
+		    CegoFieldValue *pFV = pfvl.First();
 		    while ( pFV )
 		    {
-			outRow.Insert(pFV->valAsChain());
-			pFV = fvl.Next();		    
-		    }
-		    outData.Insert(outRow);
-		    if ( count % 100 )
-		    {
-			cout << "Collecting data ... " << count << " rows\r";
-			cout.flush();
-		    }
-		}
-		else
-		{
-		    output.rowOut(fvl);
-		}
-		
-		fvl.Empty();	    
+			ListT<CegoFieldValue> fvl;			
+			CegoField *pF = schema.First();
+			while ( pF && pFV )
+			{
+			    fvl.Insert(*pFV);
+			    pF = schema.Next();
+			    pFV = pfvl.Next();
+			}
+			    
+			if ( headOut )
+			{
+			    output.headOut();
+			    headOut = false;
+			}
+			output.rowOut(fvl);
+		    }		    
+
+		    if ( hasData == false )
+			output.tailOut();
+		}			       
+		msg=Chain(count) + Chain(" tuples");
 	    }
-	    
-	    if ( outputBuffered )
+	    catch ( Exception e )
 	    {
-		ListT<Chain> pagerSchema;
-		CegoField *pF = schema.First();
-		while ( pF )
-		{
-		    Chain attrDesc = pF->getAttrName() + Chain(":") + Chain(CegoQueryHelper::maxFieldSize(pF));
-		    pagerSchema.Insert(attrDesc);		
-		    pF = schema.Next();		
-		}
-		Pager pag;
-		pag.managePage(cmd, outData, pagerSchema);
-	    }
-	    else
-	    {	    
-		output.tailOut();
-	    }
-	    msg=Chain(count) + Chain(" tuples");
-	}
-	catch ( Exception e )
-	{
-	    pCegoNet->abortQuery();
-	    e.pop(msg);
-	    exitCode=1;
+		pCegoNet->abortQuery();
+		e.pop(msg);
+		exitCode=1;
+	    }	    	  
 	}
     }
     else
@@ -1610,9 +1874,9 @@ AbortHandler::AbortHandler(const Chain& serverName,
 			   const Chain& user, 
 			   const Chain& password,
 			   unsigned long long tid, 
-			   CegoModule *pModule) :  SigHandler() 
+			   CegoModule *pModule,
+			   bool isQuiet) :  SigHandler() 
 {
-    init();
     install(SIGINT);
 #ifndef HAVE_MINGW
     install(SIGPIPE);
@@ -1624,6 +1888,7 @@ AbortHandler::AbortHandler(const Chain& serverName,
     _user = user;
     _password = password;
     _serverName = serverName;
+    _isQuiet = isQuiet;
 }
 
 AbortHandler::~AbortHandler()
@@ -1640,7 +1905,8 @@ void AbortHandler::sigCatch(int sig)
 #endif
 	if ( _doAbort )
 	{
-	    cout << endl << "Aborting query .." << endl;
+	    if ( _isQuiet == false )
+		cout << endl << "Aborting query .." << endl;
 
 	    Chain logMode("notice");
 	    if ( debug )
@@ -1665,7 +1931,8 @@ void AbortHandler::sigCatch(int sig)
 	}
 	else
 	{
-	    cout << "Use quit to quit" << endl;
+	    if ( _isQuiet == false )
+		cout << "Use quit to quit" << endl;
 	}
     }
     catch ( Exception e)
