@@ -1,19 +1,19 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoContentObject.cc 
-// ------------------
+// --------------------
 // Cego content object class implementation
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoContentObject
 // 
-// Description: This is a super class for all content classes like CegoTableObject, CegoIndexObject, and others which are used
-//              to describe data content objects.
+// Description: This is a super class for all content classes like CegoTableObject, CegoIndexObject
+//              and others which are used to describe data content objects.
 //
 // Status: CLEAN
 //
@@ -35,6 +35,7 @@ CegoContentObject::CegoContentObject(const CegoContentObject& co) : CegoDecodabl
 {
     _schema = co._schema;
     _tabName = co._tabName;
+    _tabAlias = co._tabAlias;
 }
 
 CegoContentObject::CegoContentObject(int tabSetId,
@@ -42,6 +43,7 @@ CegoContentObject::CegoContentObject(int tabSetId,
 				     const Chain& contentName) : CegoDecodableObject(type, contentName, tabSetId)
 {
     _tabName = contentName;
+    _tabAlias = contentName;
 }
 
 CegoContentObject::CegoContentObject(int tabSetId,
@@ -52,6 +54,7 @@ CegoContentObject::CegoContentObject(int tabSetId,
 {
     _schema = schema;
     _tabName = tabName;
+    _tabAlias = tabName;
 }
 
 CegoContentObject::~CegoContentObject()
@@ -90,7 +93,7 @@ ListT<CegoContentObject*>& CegoContentObject::getSubCOList()
     return _subCOList;
 }
 
-ListT<CegoField>& CegoContentObject::getSchema()
+const ListT<CegoField>& CegoContentObject::getSchema()
 {
     return _schema;
 }
@@ -114,6 +117,9 @@ int CegoContentObject::getSchemaSize() const
         // field size
 	schemaSize += sizeof(int);
 
+	// field dim
+	schemaSize += sizeof(int);
+
 	// default value size
 	schemaSize += sizeof(int) + pF->getValue().getLength();
 	
@@ -146,7 +152,7 @@ int CegoContentObject::getBaseContentSize() const
     return baseContentSize;
 }
 
-void CegoContentObject::encodeBaseContent(char *buf, int size)
+void CegoContentObject::encodeBaseContent(char *buf, int size) const
 {
     char* bufPtr = buf;
 
@@ -162,7 +168,6 @@ void CegoContentObject::encodeBaseContent(char *buf, int size)
     memcpy(bufPtr, (char*)_tabName, _tabName.length());
     bufPtr=bufPtr + _tabName.length();
 
-
     int schemaSize = getSchemaSize();
     memcpy (bufPtr, &schemaSize, sizeof(int));
     bufPtr += sizeof(int);
@@ -188,6 +193,11 @@ void CegoContentObject::encodeBaseContent(char *buf, int size)
 	memcpy (bufPtr, &l , sizeof(int));
 	bufPtr = bufPtr + sizeof(int);
 
+	int d;
+	d = pF->getDim();
+	memcpy (bufPtr, &d , sizeof(int));
+	bufPtr = bufPtr + sizeof(int);
+	
 	int deflen;
 	deflen = pF->getValue().getLength();
 	memcpy (bufPtr, &deflen , sizeof(int));
@@ -206,7 +216,6 @@ void CegoContentObject::encodeBaseContent(char *buf, int size)
 	memcpy (bufPtr, &c , 1);
 	bufPtr++;
 	
-	
 	// encode attribute
 	c = pF->getAttrName().length();
 	memcpy (bufPtr, &c , 1);
@@ -221,7 +230,6 @@ void CegoContentObject::encodeBaseContent(char *buf, int size)
 
 void CegoContentObject::decodeBaseContent(char *buf, int& size)
 {
-
     char* bufPtr = buf;
 
     CegoObject::decodeBase(bufPtr, size);
@@ -231,16 +239,18 @@ void CegoContentObject::decodeBaseContent(char *buf, int& size)
     _tabName = Chain(bufPtr);    
     bufPtr=bufPtr + _tabName.length();
 
+    // setting alias to tabname on default
+    _tabAlias = _tabName;
+    
     int schemaSize;
 
     memcpy (&schemaSize, bufPtr, sizeof(int));
     bufPtr += sizeof(int);
-
+    
     char* schemaBase = bufPtr;
     _schema.Empty();
     while ((bufPtr - schemaBase) < schemaSize)
-    {	
-
+    {
 	bool isPrimary;
 	bool isNullable;
 	char c;
@@ -262,6 +272,10 @@ void CegoContentObject::decodeBaseContent(char *buf, int& size)
 	memcpy (&len, bufPtr, sizeof(int));
 	bufPtr += sizeof(int);
 
+	int dim;
+	memcpy (&dim, bufPtr, sizeof(int));
+	bufPtr += sizeof(int);
+	
 	int deflen;
 	memcpy (&deflen, bufPtr, sizeof(int));
 	bufPtr += sizeof(int);
@@ -284,8 +298,8 @@ void CegoContentObject::decodeBaseContent(char *buf, int& size)
 	attrName = Chain(bufPtr);
 	
 	bufPtr=bufPtr + attrName.length();
-
-	_schema.Insert( CegoField( _tabName, _tabName, attrName, type, len, defValue, isNullable, id));
+	
+	_schema.Insert( CegoField( _tabName, _tabName, attrName, type, len, dim, defValue, isNullable, id));
     }
 }
 
@@ -293,6 +307,7 @@ CegoContentObject& CegoContentObject::operator = ( const CegoContentObject& co)
 {
     CegoDecodableObject::operator=(co);
     _tabName = co._tabName;
+    _tabAlias = co._tabAlias;
     _schema = co._schema;
     return (*this);
 }
