@@ -325,23 +325,52 @@ bool CegoDatabaseManager::DbSessionRecord::operator == ( const CegoDatabaseManag
 // CegoDatabaseManager //
 /////////////////////////
 
-CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& logFile, const Chain& progName, CegoDbHandler::ProtocolType protType) :  CegoBufferPool(xmlDef, logFile, progName)
+CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& lockExpire, const Chain& logFile, const Chain& progName, CegoDbHandler::ProtocolType protType) :  CegoBufferPool(xmlDef, logFile, progName)
 {
     _protType = protType;
     _lckFileName = lckFileName;
-
+    
     File lckFile(_lckFileName);
     if ( lckFile.exists() )
     {
-	Chain msg = Chain("Running database instance detected at lock file ") + lckFileName;
-	throw Exception(EXLOC, msg);
+	bool keepLock = true;
+
+	long long expire = lockExpire.asLongLong();
+	
+	if ( expire > 0 )
+	{	    
+	    if ( expire < LCKMNG_BEATDELAY )
+	    {
+		Chain msg = Chain("Lock expire too small, must be at least ") + Chain(LCKMNG_BEATDELAY) + Chain(" seconds");
+		throw Exception(EXLOC, msg);		
+	    }
+	    lckFile.open(File::READ);
+	    Chain beatString;
+	    if ( lckFile.readLine(beatString) )
+	    {
+		Datetime checkTime(beatString, DEFAULTDATETIMEFORMAT1);
+		Datetime now;
+		if ( checkTime.asLong() < now.asLong() - expire  )
+		{
+		    keepLock=false;
+		}		
+	    }
+	    lckFile.close();
+	    
+	}
+	if ( keepLock )
+	{
+	    Chain msg = Chain("Lock file ") + lckFileName + Chain(" not expired");
+	    throw Exception(EXLOC, msg);
+	}
+	else
+	{
+	    lckFile.remove();
+	}
     }
     else
     {	
-	lckFile.open(File::WRITE);	
-	lckFile.writeChain("Locked");
-	_nextBeat=0;
-	lckFile.close();
+	beat();
     }
 
     dbmLock.init(LCKMNG_LOCKWAITDELAY, __lockStatOn);  
@@ -354,7 +383,7 @@ CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFi
     _nextCopyId=1;
 
     _logConfigured=false;
-    
+
     _modId = getModId("CegoDatabaseManager");
 }
 
@@ -482,8 +511,8 @@ void CegoDatabaseManager::beat()
 {
     File lckFile(_lckFileName);
     lckFile.open(File::WRITE);
-    lckFile.writeChain(Chain("Beat=") + Chain(_nextBeat) + Chain("\n"));	
-    _nextBeat++;
+    Datetime dt;
+    lckFile.writeChain(dt.asChain(DEFAULTDATETIMEFORMAT1) + Chain("\n"));
     lckFile.close();    
 }
 
@@ -661,6 +690,7 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
             throw Exception(EXLOC, msg); 
         }
 	
+	// cout << "Using obj " << objName << " numused = " << pOR->numUsed() << endl;
 	if ( pOR->numUsed() > MAX_OBJECT_USAGE )
 	{
             V();
