@@ -33,7 +33,7 @@ CegoDistCursor::CegoDistCursor()
 {
 }
 
-CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO)
+CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO, CegoProcBlock *pBlock)
 {
     _isLocal = true;
     _moreTuple = false;
@@ -47,6 +47,8 @@ CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO)
     _tableName = pCO->getTabName();
     _tableAlias = pCO->getName();
     _tabSetId = pCO->getTabSetId();
+
+    _pBlock = pBlock;
     
     _pDBMng = _pGTM->getDBMng(); 
 
@@ -66,6 +68,9 @@ CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO)
 
     _pFLA = 0;
     _pFLAmap = 0;
+
+    _pPropPred = 0;
+    _pJoinPred = 0;
     
     _idxMatch = CegoAttrCond::INAPP;
 
@@ -132,6 +137,11 @@ CegoDistCursor::~CegoDistCursor()
 	delete _pC;
     }
 
+    if ( _pJoinPred )
+    {
+	delete _pJoinPred;
+    }
+	    
     try
     {
 	unuseCursorObject();
@@ -256,6 +266,7 @@ void CegoDistCursor::checkType()
 
 	// since a view might be used several times inside a single query, we have to use a clone	
 	_pSelect = pView->getSelect()->clone(false);
+	_pSelect->setProcBlock(_pBlock);
 	_pSelect->cleanUp();
     }
     else if ( _pCO->getType() == CegoObject::ALIAS )
@@ -267,7 +278,7 @@ void CegoDistCursor::checkType()
 	_pTO = new CegoTableObject();
 	_pGTM->getDistObject(tableSet, _tableName, CegoObject::TABLE, *_pTO);
 
-	_pTCTab = new CegoDistCursor(_pGTM, _pTO);	
+	_pTCTab = new CegoDistCursor(_pGTM, _pTO, _pBlock);	
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
@@ -319,18 +330,81 @@ void CegoDistCursor::checkType()
 	CegoContentObject *pCOLeft = pJCO->getLeftObject();
 	CegoContentObject *pCORight = pJCO->getRightObject();
 	
-	_pTCLeft = new CegoDistCursor(_pGTM, pCOLeft);
-	_pTCRight = new CegoDistCursor(_pGTM, pCORight);
+	_pTCLeft = new CegoDistCursor(_pGTM, pCOLeft, _pBlock);
+	_pTCRight = new CegoDistCursor(_pGTM, pCORight, _pBlock);
     }
 
     // we have to release object here to avoid dead locks
     unuseCursorObject();    
 }
 
-void CegoDistCursor::distSetup( ListT<CegoField>** pFLA)
+void CegoDistCursor::distSetup(ListT<CegoField>** pFLA, CegoPredicate* pPred)
 {
+    
     _isFirst=true;
     _doEval = false;
+
+    if ( _pJoinPred )
+    {
+	delete _pJoinPred;
+	_pJoinPred = 0;
+    }
+    
+    if (  pPred &&  ( _pCO->getType() == CegoObject::TABLE || _pCO->getType() == CegoObject::VIEW  || _pCO->getType() == CegoObject::ALIAS)  )
+    {
+	_pPropPred = pPred->getReduced(_tableAlias);
+	_pJoinPred = _pPropPred;
+    }
+    else if ( pPred && _pCO->getType() == CegoObject::JOIN )
+    {
+	// set up for further propagation
+	_pPropPred = pPred;
+	
+	CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
+	
+	if ( pJO->getJoinType() == CegoJoinObject::INNER )
+	{
+	    Chain leftAlias = pJO->getLeftObject()->getTabAlias();
+
+	    ListT<Chain> aliasList;
+	    if ( pJO->getLeftObject()->getType() != CegoObject::JOIN )
+		aliasList.Insert(leftAlias);
+	    else
+	    {
+		ListT<Chain> subList = pJO->getSubAliasList();
+		aliasList += subList;
+	    }
+	    
+	    
+	    Chain rightAlias = pJO->getRightObject()->getTabAlias();
+	    if ( pJO->getRightObject()->getType() != CegoObject::JOIN )
+		aliasList.Insert(rightAlias);
+
+	    // cout << "1 Setting up cursor " << _tableAlias << " pred for " << leftAlias << " and " << rightAlias << endl;
+	    _pJoinPred = pPred->getReduced(aliasList);
+	    
+	}
+    }
+    else
+    {	
+	_pPropPred = 0;
+	_pJoinPred = 0;
+    }
+
+
+#ifdef CGDEBUG
+    Chain predCond;
+    if ( _pJoinPred )
+	predCond = _pJoinPred->toChain(0);
+    _pDBMng->log(_modId, Logger::DEBUG, Chain("Setting empty condition and master condition <") + predCond + Chain("> for ") + _tableAlias);	
+#endif
+
+
+    // if could be reduced, setup propagated predicate to checked
+    if ( pPred )
+	if ( _pJoinPred )
+	    pPred->setChecked(true);
+    
     _idxMatch = CegoAttrCond::INAPP;
     _isAttrCondValid = false;
     _pFLA = pFLA;
@@ -339,14 +413,14 @@ void CegoDistCursor::distSetup( ListT<CegoField>** pFLA)
     {
 	_pSelect->setTabSetId(_tabSetId);
 	CegoAttrCond noCond;
-
-	_pSelect->setViewCond(noCond, pFLA);
+	
+	_pSelect->setViewCond(_pCO->getTabAlias(), noCond, _pPropPred, pFLA);
 	_pSelect->prepare();
 	_pSelect->checkValidRef();
     }
     else if ( _pCO->getType() == CegoObject::ALIAS )
     {
-	_pTCTab->distSetup(_pFLA);
+	_pTCTab->distSetup(_pFLA, _pPropPred);
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {	
@@ -387,17 +461,68 @@ void CegoDistCursor::distSetup( ListT<CegoField>** pFLA)
     }
 }
 
-void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA)
-{
-    
+void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA, CegoPredicate* pPred)
+{    
     if ( attrCond.numComp() == 0 )
     {
-	return distSetup(pFLA);
+	return distSetup(pFLA, pPred);	
     }
+    
+    if ( _pJoinPred )
+    {
+	delete _pJoinPred;
+	_pJoinPred = 0;
+    }
+
     _isAttrCondValid = false;
     _idxMatch = CegoAttrCond::INAPP;
     _isFirst=true;
     _doEval = false;
+
+    if (  pPred &&  ( _pCO->getType() == CegoObject::TABLE || _pCO->getType() == CegoObject::VIEW  || _pCO->getType() == CegoObject::ALIAS)  )
+    {
+	// cout << "Reducing " << pPred->toChain(0) << " for " << _tableAlias << endl;
+	_pPropPred = pPred->getReduced(_tableAlias);
+	// if ( _pPropPred )
+	//     cout << " => " << _pPropPred->toChain(0) << endl;
+	_pJoinPred = _pPropPred;
+    }
+    else if ( pPred && _pCO->getType() == CegoObject::JOIN )
+    {	
+	// set up for further propagation
+	_pPropPred = pPred;
+	
+	CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
+	
+	if ( pJO->getJoinType() == CegoJoinObject::INNER )
+	{
+	    Chain leftAlias = pJO->getLeftObject()->getTabAlias();
+
+	    ListT<Chain> aliasList;
+	    if ( pJO->getLeftObject()->getType() != CegoObject::JOIN )
+		aliasList.Insert(leftAlias);
+	    
+	    Chain rightAlias = pJO->getRightObject()->getTabAlias();
+	    if ( pJO->getRightObject()->getType() != CegoObject::JOIN )
+		aliasList.Insert(rightAlias);
+
+	    // cout << "2 Setting up cursor " << _tableAlias << " pred for " << leftAlias << " and " << rightAlias << endl;
+	    _pJoinPred = pPred->getReduced(aliasList);
+
+	}
+    }
+    else
+    {
+	_pPropPred = 0;
+	_pJoinPred = 0;
+    }
+
+
+    // if could be reduced, setup propagated predicate to checked
+    if ( pPred )
+	if ( _pJoinPred )
+	    pPred->setChecked(true);
+
     _pFLA = pFLA;
     
     if ( _pCO->getType() == CegoObject::VIEW )
@@ -415,11 +540,24 @@ void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA)
 	// if cannot be setup completely ( aggregation in expression ),
 	// we force a dedicated evaluation
 
-	if ( _pSelect->setViewCond(attrCond, pFLA) )
+	/*
+	if ( _pPred )
+	    cout << "2. Setting view cond " << _pPred->toChain(0) << " for " << _tableAlias << endl;
+	else
+	    cout << "2. No condition for view " << _tableAlias << endl;
+	*/
+	
+	if ( _pSelect->setViewCond(_tableAlias, attrCond, _pPropPred, pFLA) )
 	    _doEval=false;
 	else
 	    _doEval=true;
 
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting cursor condition eval =  ")
+		     + Chain(_doEval));
+#endif
+
+	
 	_pSelect->prepare();
 
 	_pSelect->checkValidRef();
@@ -431,10 +569,15 @@ void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA)
 	CegoAliasObject *pAO = (CegoAliasObject*)_pCO;
 	CegoAttrCond mapAttrCond = CegoQueryHelper::mapAttrCond(attrCond, pAO);
 	
-	_pTCTab->distSetup(mapAttrCond, pFLA);
+	_pTCTab->distSetup(mapAttrCond, pFLA, _pPropPred);
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
+
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting table condition ") + attrCond.toChain());	
+#endif
+
 	if ( _isLocal == true )
 	{
 	    if ( _pCache )
@@ -449,6 +592,11 @@ void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA)
 	    
 	    if ( _idxMatch != CegoAttrCond::FULL )
 	    {
+
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting val to true "));	
+#endif
+			
 		_doEval=true;
 
 		// if index match was inappropriate, all tuples are retrieved from table cursor
@@ -473,7 +621,7 @@ void CegoDistCursor::distSetup( CegoAttrCond& attrCond, ListT<CegoField>** pFLA)
 		    }
 		}
 	    }
-	    _cursorCond = attrCond;  
+	    _cursorCond = attrCond;	    
 	}
     }
     else if ( _pCO->getType() == CegoObject::SYSTEM )
@@ -506,8 +654,9 @@ bool CegoDistCursor::nextTuple(ListT<CegoField>** pFLA, int pos, int size)
     
     while ( getTuple(pFLA, pos, size) )
     {
+	
 	/*
-	cout << "### TUPLE = ";
+	cout << "### " << _tableAlias << " TUPLE = ";
 	for ( int i=0; i<size; i++)
 	{
 	    
@@ -520,125 +669,177 @@ bool CegoDistCursor::nextTuple(ListT<CegoField>** pFLA, int pos, int size)
 	}
 	cout << endl;
 	*/
-	
+
+	bool evaluated=true;
+
+	// first step : if required, evaluate join condition 
 	if ( _doEval == true )
 	{
-	    if ( evalCondition(pFLA, pos, size) )
-	    {
-#ifdef CGDEBUG
-		_pDBMng->log(_modId, Logger::DEBUG, Chain("Returning tuple with eval condition"));
-#endif
-		return true;
-	    }
+	    evaluated =  evalCondition(pFLA, pos, size);
 	}
-	else
+	
+	// cout << "Eval join pred ? " << endl; 
+	// second step : if available, evaluate propagated master predicate
+	if ( evaluated && _pJoinPred )
 	{
-#ifdef CGDEBUG
-	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Returning tuple without eval condition"));
-#endif
-	    return true;
+	    // cout << "Eval  " << _pJoinPred->toChain(0) << " for " << _tableAlias << endl; 
+	    evaluated =  _pJoinPred->eval(_pFLA,
+					  0,
+					  pFLA,
+					  0,							       
+					  _pBlock);
 	}
+	
+	if ( evaluated )
+	{
+	    
+#ifdef CGDEBUG
+	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Cursor ") + _tableAlias + Chain(" returns tuple"));
+#endif	    
+	    return true;
+	}    
     }
     return false;
 }
 
 bool CegoDistCursor::evalCondition(ListT<CegoField>** pFLA, int pos, int size)
 {
-    CegoAttrComp *pAC = _cursorCond.getAttrCompSet().First();
     
-    while ( pAC )
+    if ( _cursorCond.numComp() == 0 )
+	return true;
+    
+    bool isAnd = _cursorCond.isAnd();
+    
+#ifdef CGDEBUG
+    _pDBMng->log(_modId, Logger::DEBUG, Chain("Starting eval condition for cursorcond ") + _cursorCond.toChain());
+#endif	    
+
+    bool validFound = false;
+
+    CegoAttrComp *pAC = _cursorCond.getAttrCompSet().First();
+    while ( pAC  )
     {
-	// cout << "Eval condition " << *pAC << endl;	
-	// cout << "Condition " << pAC->getTableName() << " . " <<  pAC->getAttrName() << endl;
+
+	validFound =  evalCompare(pAC, pFLA, pos, size);
+    
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Eval ") + pAC->toChain() + Chain(" = ") + Chain(validFound));
+#endif	    
 	
-	CegoField *pF = 0;
-	    
-	int i=0;
-	while ( pF == 0 && i < size)
-	{
-	    if ( _pCO->getType() == CegoObject::VIEW || _pCO->getType() == CegoObject::TABLE )
-		pF = pFLA[pos+i]->Find( CegoField(_tableAlias, pAC->getAttrName()));
-	    else
-		pF = pFLA[pos+i]->Find( CegoField(pAC->getTableName(), pAC->getAttrName()) );
-	    i++;
-	}
+	if ( validFound == false && isAnd == true )
+	    return false;
+	if ( validFound == true && isAnd == false)
+	    return true;
 	
-	if ( pF )
-	{
-	    if ( pAC->getCompMode() == CegoAttrComp::BTWN )
-	    {
-		if ( ( (CegoFieldValue)pF->getValue() >= (CegoFieldValue)(pAC->getFieldValue()) 
-		       && (CegoFieldValue)pF->getValue() <= (CegoFieldValue)(pAC->getFieldValue2()) ) == false )
-		    
-		    return false;	    
-	    }
-	    else if ( pAC->getCompMode() == CegoAttrComp::ISLIKE )
-	    {
-		if ( pAC->getMatcher()->match(pF->getValue().valAsChain()) == false)		
-		    return false;
-	       
-	    }
-	    else if ( pAC->getCompMode() == CegoAttrComp::ISNOTLIKE )
-	    {
-		if ( pAC->getMatcher()->match(pF->getValue().valAsChain()))
-		    return false;		
-	    }
-	    else
-	    {		
-		switch ( pAC->getComparison() )
-		{	    
-		case EQUAL:
-		{
-		    // cout << "Eval " << pF->getValue() << " = " << pAC->getFieldValue() << endl;
-		    if ( ( (CegoFieldValue)pF->getValue() == (CegoFieldValue)(pAC->getFieldValue()) ) == false )		    
-			return false;		    
-		    break;
-		}
-		case NOT_EQUAL:
-		{
-		    if ( ( (CegoFieldValue)pF->getValue() != (CegoFieldValue)(pAC->getFieldValue()) ) == false )		    
-			return false;	    
-		    break;
-		}
-		case LESS_THAN:
-		{
-		    if ( ( (CegoFieldValue)pF->getValue() < (CegoFieldValue)(pAC->getFieldValue()) ) == false )		     
-			return false;
-		    break;
-		}
-		case MORE_THAN: 
-		{
-		    if ( ( (CegoFieldValue)pF->getValue() > (CegoFieldValue)(pAC->getFieldValue()) ) == false )
-			return false;
-		    break;
-		}
-		case LESS_EQUAL_THAN:
-		{
-		    if ( ( (CegoFieldValue)pF->getValue() <= (CegoFieldValue)(pAC->getFieldValue()) ) == false )
-			return false;
-		    break;
-		}
-		case MORE_EQUAL_THAN:
-		{
-		    if ( ( (CegoFieldValue)pF->getValue() >= (CegoFieldValue)(pAC->getFieldValue()) ) == false )
-			return false;	    
-		    break;
-		}
-		}
-	    }
-	}
-	else
-	{
-	    if ( pAC->isSetup() == false )
-	    {
-		Chain msg = Chain("Cannot get value for attribute ") + pAC->getAttrName();
-		throw Exception(EXLOC, msg);
-	    }
-	}
 	pAC = _cursorCond.getAttrCompSet().Next();
+    }    
+    
+    return validFound;
+}
+
+bool CegoDistCursor::evalCompare(CegoAttrComp *pAC, ListT<CegoField>** pFLA, int pos, int size)
+{
+
+    // cout << "Eval compare " << *pAC << endl;	
+    // cout << "Condition " << pAC->getTableName() << " . " <<  pAC->getAttrName() << endl;
+    
+    CegoField *pF = 0;
+    
+    int i=0;
+    while ( pF == 0 && i < size)
+    {
+	if ( _pCO->getType() == CegoObject::VIEW || _pCO->getType() == CegoObject::TABLE )
+	    pF = pFLA[pos+i]->Find( CegoField(_tableAlias, pAC->getAttrName()));
+	else
+	    pF = pFLA[pos+i]->Find( CegoField(pAC->getTableName(), pAC->getAttrName()) );
+	i++;
+    }
+    
+    if ( pF )
+    {	    
+	if ( pAC->getCompMode() == CegoAttrComp::BTWN )
+	{
+	    if ( ( (CegoFieldValue)pF->getValue() >= (CegoFieldValue)(pAC->getFieldValue()) 
+		   && (CegoFieldValue)pF->getValue() <= (CegoFieldValue)(pAC->getFieldValue2()) ) == false )
+		
+		return false;	    
+	}
+	else if ( pAC->getCompMode() == CegoAttrComp::ISLIKE )
+	{
+	    if ( pAC->getMatcher()->match(pF->getValue().valAsChain()) == false)		
+		return false;
+	}
+	else if ( pAC->getCompMode() == CegoAttrComp::ISNOTLIKE )
+	{
+	    if ( pAC->getMatcher()->match(pF->getValue().valAsChain()))
+		return false;		
+	}
+	else if ( pAC->getCompMode() == CegoAttrComp::ISNCLIKE )
+	{
+	    if ( pAC->getMatcher()->match(pF->getValue().valAsChain().toLower()) == false)		
+		return false;	    
+	}
+	else if ( pAC->getCompMode() == CegoAttrComp::ISNOTNCLIKE )
+	{
+	    if ( pAC->getMatcher()->match(pF->getValue().valAsChain().toLower()))
+		return false;		
+	}
+	
+	else
+	{		
+	    switch ( pAC->getComparison() )
+	    {	    
+	    case EQUAL:
+	    {
+		// cout << "Eval " << pF->getValue() << " = " << pAC->getFieldValue() << endl;
+		if ( ( (CegoFieldValue)pF->getValue() == (CegoFieldValue)(pAC->getFieldValue()) ) == false )		    
+		    return false;
+		break;
+	    }
+	    case NOT_EQUAL:
+	    {
+		if ( ( (CegoFieldValue)pF->getValue() != (CegoFieldValue)(pAC->getFieldValue()) ) == false )		    
+		    return false;	    
+		break;
+	    }
+	    case LESS_THAN:
+	    {
+		if ( ( (CegoFieldValue)pF->getValue() < (CegoFieldValue)(pAC->getFieldValue()) ) == false )		     
+		    return false;
+		break;
+	    }
+	    case MORE_THAN: 
+	    {
+		if ( ( (CegoFieldValue)pF->getValue() > (CegoFieldValue)(pAC->getFieldValue()) ) == false )
+		    return false;
+		break;
+	    }
+	    case LESS_EQUAL_THAN:
+	    {
+		if ( ( (CegoFieldValue)pF->getValue() <= (CegoFieldValue)(pAC->getFieldValue()) ) == false )
+		    return false;
+		break;
+	    }
+	    case MORE_EQUAL_THAN:
+	    {
+		if ( ( (CegoFieldValue)pF->getValue() >= (CegoFieldValue)(pAC->getFieldValue()) ) == false )
+		    return false;	    
+		break;
+	    }
+	    }
+	}
+    }
+    else
+    {
+	if ( pAC->isSetup() == false )
+	{
+	    Chain msg = Chain("Cannot get value for attribute ") + pAC->getAttrName();
+	    throw Exception(EXLOC, msg);
+	}
     }
     return true;
 }
+    
 
 bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 {
@@ -992,7 +1193,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	return _moreTuple;
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
-    {	
+    {		
 	if ( _isFirst )
 	{
 	    _moreLeft = true;
@@ -1009,22 +1210,25 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	    {
 		if ( _isFirst )
 		{
-		    _pTCLeft->distSetup(_outerCond, _pFLA);
+		    // cout << "OuterCond=" << _outerCond.toChain() << endl;
+		    _pTCLeft->distSetup(_outerCond, _pFLA, _pPropPred);
 		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    if ( _moreLeft )
-		    {			
+		    {
+			// cout << "InnerCond=" << _innerCond.toChain() << endl;
 			_innerCond.setup(pFLA, offset);
 
 			_pTCRight->reset();
 			
 			if ( _isAttrCondValid )
 			{
-			    _pTCRight->distSetup(_innerCond, _pFLA);
+			    _pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 			}
 			else
 			{
-			    _pTCRight->distSetup(_pFLA);
+			    _pTCRight->distSetup(_pFLA, _pPropPred);
 			}
+			
 			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
 		    _isFirst = false;
@@ -1042,11 +1246,11 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 
 			    if ( _isAttrCondValid )
 			    {
-				_pTCRight->distSetup(_innerCond, _pFLA);
+				_pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 			    }
 			    else
 			    {
-				_pTCRight->distSetup(_pFLA);
+				_pTCRight->distSetup(_pFLA, _pPropPred);
 			    }
 			    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 			}
@@ -1062,11 +1266,11 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 
 			if ( _isAttrCondValid )
 			{
-			    _pTCRight->distSetup(_innerCond, _pFLA);
+			    _pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 			}
 			else
 			{
-			    _pTCRight->distSetup(_pFLA);
+			    _pTCRight->distSetup(_pFLA, _pPropPred);
 			}
 			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
@@ -1082,13 +1286,26 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			isEvaluated = pJO->getPredDesc()->eval(0,
 							       0,
 							       pFLA,
-							       offset,							       
-							       0); 
+							       offset,
+							       _pBlock);
 		    }
 		    else
 		    {			
 			isEvaluated = true;
 		    }
+
+		  		   
+		    if ( isEvaluated && _pJoinPred )
+		    {
+
+			isEvaluated = _pJoinPred->eval(_pFLA,
+						       0,
+						       pFLA,
+						       0,							       
+						       _pBlock);    			
+		    }
+		    
+		    
 		}
 	    }
 	    return _moreLeft && _moreRight;
@@ -1096,8 +1313,8 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	else if ( pJO->getJoinType() == CegoJoinObject::LEFTOUTER )
 	{
 	    if ( _isFirst )
-	    {
-		_pTCLeft->distSetup(_outerCond, _pFLA);	    
+	    {	
+		_pTCLeft->distSetup(_outerCond, _pFLA, _pPropPred);	    
 		_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		
 		if ( _moreLeft )
@@ -1107,11 +1324,11 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    
 		    if ( _isAttrCondValid )
 		    {
-			_pTCRight->distSetup(_innerCond, _pFLA);
+			_pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 		    }
 		    else
 		    {
-			_pTCRight->distSetup(_pFLA);
+			_pTCRight->distSetup(_pFLA, _pPropPred);
 		    }
 		    
 		    nextRight(pFLA, offset, size);
@@ -1133,11 +1350,11 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			
 			if ( _isAttrCondValid )
 			{
-			    _pTCRight->distSetup(_innerCond, _pFLA);
+			    _pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 			}
 			else
 			{
-			    _pTCRight->distSetup(_pFLA);
+			    _pTCRight->distSetup(_pFLA, _pPropPred);
 			}
 
 			nextRight(pFLA, offset, size);
@@ -1155,11 +1372,11 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    
 		    if ( _isAttrCondValid )
 		    {
-			_pTCRight->distSetup(_innerCond, _pFLA);
+			_pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 		    }
 		    else
 		    {
-			_pTCRight->distSetup(_pFLA);
+			_pTCRight->distSetup(_pFLA, _pPropPred);
 		    }
 
 		    nextRight(pFLA, offset, size);
@@ -1187,7 +1404,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 	{    
 	    if ( _isFirst )
 	    {		
-		_pTCRight->distSetup(_outerCond, _pFLA);
+		_pTCRight->distSetup(_outerCond, _pFLA, _pPropPred);
 		
 		_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		
@@ -1197,9 +1414,9 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    _pTCLeft->reset();
 		    
 		    if ( _isAttrCondValid )
-			_pTCLeft->distSetup(_innerCond, _pFLA);
+			_pTCLeft->distSetup(_innerCond, _pFLA, _pPropPred);
 		    else
-			_pTCLeft->distSetup(_pFLA);
+			_pTCLeft->distSetup(_pFLA, _pPropPred);
 
 		    nextLeft(pFLA, offset, size);
 		}
@@ -1219,9 +1436,9 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 			_pTCLeft->reset();
 			
 			if ( _isAttrCondValid )
-			    _pTCLeft->distSetup(_innerCond, _pFLA);
+			    _pTCLeft->distSetup(_innerCond, _pFLA, _pPropPred);
 			else
-			    _pTCLeft->distSetup(_pFLA);
+			    _pTCLeft->distSetup(_pFLA, _pPropPred);
 
 			nextLeft(pFLA, offset, size);
 		    }
@@ -1237,9 +1454,9 @@ bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 		    _pTCLeft->reset();
 		    
 		    if ( _isAttrCondValid )
-			_pTCLeft->distSetup(_innerCond, _pFLA);
+			_pTCLeft->distSetup(_innerCond, _pFLA, _pPropPred);
 		    else
-			_pTCLeft->distSetup(_pFLA);
+			_pTCLeft->distSetup(_pFLA, _pPropPred);
 
 		    nextLeft(pFLA, offset, size);
 		}	   	
@@ -1278,16 +1495,35 @@ void CegoDistCursor::nextRight(ListT<CegoField>** pFLA, int offset, int size)
     _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
     
     bool notFound = true;
-    while ( _evalPredicate && _moreRight && notFound )
+    while ( ( _evalPredicate || _pJoinPred ) && _moreRight && notFound )
     {
 	// attr cache is left for performace
 	// pJO->getPredDesc()->clearAttrCache();
+
 	
-	if ( pJO->getPredDesc()->eval(0,
-				      0,
-				      pFLA,
-				      offset,
-				      0) )
+	bool isValid = true;
+
+	if ( _evalPredicate )
+	{
+	    isValid =  pJO->getPredDesc()->eval(0,
+						0,
+						pFLA,
+						offset,
+						_pBlock);
+	}
+	
+	if ( isValid && _pJoinPred )
+	{
+
+	    isValid = _pJoinPred->eval(_pFLA,
+				       0,
+				       pFLA,
+				       0,							       
+				       _pBlock);
+	}
+	
+	
+	if ( isValid )    
 	{
 	    notFound = false;
 	}
@@ -1305,14 +1541,32 @@ void CegoDistCursor::nextLeft(ListT<CegoField>** pFLA, int offset, int size)
     _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
     
     bool notFound = true;
-    while ( _evalPredicate && _moreLeft && notFound )
+    while ( ( _evalPredicate || _pJoinPred )  && _moreLeft && notFound )
     {
 	
-	if ( pJO->getPredDesc()->eval(0,
-				      0,
-				      pFLA,
-				      offset,
-				      0) )
+	bool isValid =  true;
+
+	if ( _evalPredicate )
+	{
+	    isValid = pJO->getPredDesc()->eval(0,
+					       0,
+					       pFLA,
+					       offset,
+					       _pBlock);
+	}
+
+       
+	if ( isValid && _pJoinPred )
+	{
+	    isValid = _pJoinPred->eval(_pFLA,
+				       0,
+				       pFLA,
+				       0,							       
+				       _pBlock);	    			    
+	}
+	
+
+	if ( isValid ) 
 	{
 	    notFound = false;
 	}
@@ -1370,7 +1624,8 @@ void CegoDistCursor::reset()
 
     if ( _pCO->getType() == CegoObject::VIEW )
     {
-	_pSelect->reset(true);
+	// was true
+	_pSelect->reset();
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
@@ -1468,15 +1723,19 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
 	if ( pJO->getJoinType() == CegoJoinObject::INNER )
 	{
 	    _outerCond = attrCond.getFilterCond(outerSchema, false);
+
+	    // cout << "AttrCond = " << attrCond.toChain() << endl;
 	    _innerCond = attrCond.getFilterCond(innerSchema, false);
 
+	    // cout << "InnerCond = " << _innerCond.toChain() << endl;
 	    _doEval = false;
 	}
 	else
-	{
-	    _outerCond = attrCond.getFilterCond(outerSchema, true);
+	{	    
+	    _outerCond = attrCond.getFilterCond(outerSchema, false);
 	    _innerCond = attrCond.getFilterCond(innerSchema, true);
 
+	    // for left outer join, we have to evaluate the where condition outside the join 
 	    _doEval = true;
 	}    
     }
@@ -1484,15 +1743,15 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
     {
 	innerSchema = pJO->getLeftObject()->getSchema();
         outerSchema = pJO->getRightObject()->getSchema();
-
-	_outerCond = attrCond.getFilterCond(outerSchema, true);
+	
+	_outerCond = attrCond.getFilterCond(outerSchema, false);
 	_innerCond = attrCond.getFilterCond(innerSchema, true);
 
 	// for right outer join, we have to evaluate the where condition outside the join 
 	_doEval = true;	
     }
 
-    // cout << "Setting up join with " << pJO->getPredDesc()->toChain() << endl;
+    // cout << "Setting up join with " << pJO->getPredDesc()->toChain(0) << endl;
 
     ListT<CegoPredicate*> conjunctionList;
 
@@ -1504,8 +1763,6 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
     while ( pPred )
     {
 	CegoAttrCond ac;
-	// CegoQueryHelper::AttrCondMatch m  = CegoQueryHelper::checkAttrCond(ac, pJO->getPredDesc(), innerSchema, &outerSchema, 1, 0);
-
 	CegoQueryHelper::AttrCondMatch m  = CegoQueryHelper::checkAttrCond(ac, *pPred, innerSchema, &outerSchema, 1, 0);
 
 	if ( m == CegoQueryHelper::COMPLETE )
@@ -1526,6 +1783,8 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
 	pPred = conjunctionList.Next();
     }
 
+    // cout << "InnerCond = " << _innerCond.toChain() << " eval=" << _evalPredicate << endl;
+
     _isAttrCondValid = true;    
 }
 
@@ -1573,21 +1832,32 @@ Element* CegoDistCursor::getPlan()
     {
 	pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_TABLE_VALUE);
 	
+	Chain joinStrat;
 	if ( _idxMatch == CegoAttrCond::FULL)
 	{
-	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("full index trace on ") + _cursorCond.toChain()); 
+	    joinStrat = Chain("full index trace on ") + _cursorCond.toChain(); 
 	}
 	else if( _idxMatch == CegoAttrCond::PART )
 	{
-	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("index support on ") + _cursorCond.toChain() + Chain(" using index ") + _pTC->getIndexName()); 
+	    joinStrat =  Chain("index support on ") + _cursorCond.toChain() + Chain(" using index ") + _pTC->getIndexName(); 
 	}
 	else
-	{
+	{	    	    
 	    if ( _cursorCond.numComp() > 0 )
-		pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("full table scan using condition ") + _cursorCond.toChain());
+	    {
+		joinStrat = Chain("full table scan using condition ") + _cursorCond.toChain();		
+	    }
 	    else
-		pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("full table scan with no condition "));
-	}	
+	    {
+		joinStrat = Chain("full table scan with no condition ");
+	    }
+	}
+		
+	if ( _pJoinPred )
+	    joinStrat += Chain(" on ") + _tableAlias + Chain(" with master predicate ") + _pJoinPred->toChain(0);
+	
+	pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, joinStrat);
+	
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {
@@ -1600,24 +1870,38 @@ Element* CegoDistCursor::getPlan()
 	else if ( pJCO->getJoinType() == CegoJoinObject::RIGHTOUTER)	    
 	    pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_RIGHTOUTERJOIN_VALUE);
 
-	if ( _evalPredicate )	    
-	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("with condition evaluation"));
+	Chain joinStrat;
+	
+	if ( _evalPredicate )
+	{
+	    CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
+	    Chain joinPred = pJO->getPredDesc()->toChain(0);	    
+	    joinStrat = Chain("join condition ") + joinPred;
+	}
 	else
-	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("no condition evaluation"));
+	    joinStrat = Chain("no join condition");
+	
+	if ( _pJoinPred )
+	{
+	    joinStrat += Chain(" and master predicate ") + _pJoinPred->toChain(0);
+	}
+	
+
+	pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, joinStrat);
 
 	if ( _pTCLeft )
 	{
-	    _pTCLeft->distSetup(_outerCond, _pFLA);
+	    _pTCLeft->distSetup(_outerCond, _pFLA, _pPropPred);
 	}
 	if ( _pTCRight )
 	{	    
 	    if ( _isAttrCondValid )
 	    {
-		_pTCRight->distSetup(_innerCond, _pFLA);
+		_pTCRight->distSetup(_innerCond, _pFLA, _pPropPred);
 	    }
 	    else
 	    {
-		_pTCRight->distSetup(_pFLA);
+		_pTCRight->distSetup(_pFLA, _pPropPred);
 	    }	    
 	}
 	if ( _pTCLeft )
@@ -1630,6 +1914,7 @@ Element* CegoDistCursor::getPlan()
 	delete pCursorPlan;
 	throw Exception(EXLOC, Chain("Invalid content type"));		
     }
+    
 
     return pCursorPlan;
 }
