@@ -266,6 +266,12 @@ void CegoDistManager::startTableSet(const Chain& tableSet, const Chain& secondar
     {
 	throw Exception(EXLOC, Chain("Checkpoint crash detected, tableset might be inconsistent")); 	
     }
+
+    Chain tsTicketName = _pDBMng->getTSTicket(tableSet);
+    
+    File tsTicket(tsTicketName);
+    if ( tsTicket.exists()) 
+	throw Exception(EXLOC, Chain("Backup tableset ticket exists"));
     
     int tabSetId = _pDBMng->getTabSetId(tableSet);
     regDataFiles(tableSet);
