@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -42,67 +42,71 @@ public:
    
     enum FileType { DATAFILE, SYSTEMFILE, TEMP };
 
-    CegoFileHandler(const Chain& logFile, const Chain& progName, int pageSize = 16384);
+    CegoFileHandler(const Chain& logFile, const Chain& progName);
     ~CegoFileHandler();
 
-    bool isBackup(int fileId);
+    void setReadOnly(bool isReadOnly);
+    
     void setBackup(int fileId, bool m);
-    bool isMarked(int fileId, int pageId);
-        
-    void cleanDataFile(int fileId, CegoLockHandler* pLockHandle);
 
-    void initDataFile(int tabSetId, const Chain& path, int fileId, int numPages, FileType ft);
+    bool needPageBackup(PageIdType pageId);
+    bool isPageMarked(PageIdType pageId, int fileId);
+
+    void initDataFile(int tabSetId, const Chain& path, int fileId, int numPages, PageIdType pageOffset, FileType ft);
     void regDataFile(int tabSetId, const Chain& path, int fileId, CegoLockHandler* pLockHandle);
 
-    void writePage(int fileId, int pageId, int fixStat, char* pageData, CegoLockHandler* pLockHandle);
-    void readPage(int fileId, int pageId, int& tabSetId, int& fixStat, char* pageData, CegoLockHandler* pLockHandle);
+    void resetDataFile(int fileId);
+	
+    void writePage(PageIdType pageId, unsigned fixStat, char* pageData, CegoLockHandler* pLockHandle);
+    void readPage(PageIdType pageId, int& tabSetId, unsigned& fixStat, char* pageData, CegoLockHandler* pLockHandle);
     
-    void claimPage(int fileId, int pageId, CegoLockHandler* pLockHandle);
-    void allocatePage(int tabSetId, FileType ft, int& fileId, int& pageId, CegoLockHandler* pLockHandle, unsigned* &fbm, int& fbmSize, bool doAppend = false);
-    void releasePage(int fileId, int pageId, CegoLockHandler* pLockHandle, unsigned* &fbm, int& fbmSize);
+    void claimPage(PageIdType pageId, CegoLockHandler* pLockHandle);
+    void allocatePage(int tabSetId, FileType ft, PageIdType& pageId, CegoLockHandler* pLockHandle, unsigned* &fbm, int& fbmSize, bool doAppend = false);
+    void releasePage(PageIdType pageId, CegoLockHandler* pLockHandle, unsigned* &fbm, int& fbmSize);
 
     void releaseFiles(int tabSetId);
 
     int getNumPages(int fileId);
     int getNumUsedPages(int fileId, CegoLockHandler* pLockHandle);
 
-    unsigned long long getRegPageOffset(int fileId);
+    PageIdType getPageOffset(int fileId);
+    
+    bool isClaimed(PageIdType pageId, CegoLockHandler *pLockHandle);
     
-    int getFBMSize(int fileId);
-    void readFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockHandle);
     void writeFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockHandle);
 
-    bool isClaimed(int fileId, int pageId, CegoLockHandler *pLockHandle);
-    
-    void printBitMap(unsigned bm);
+    int getFileIdForPageId(PageIdType pageId);
 
+    void commitPageEntry(PageIdType pageId);
+    int cleanPages(CegoLockHandler *pLockHandle);
+    
 protected:
     
     int _pageSize;
 
 private:
-    
-    void markPage(int fileId, int pageId);
 
+    int getFBMSize(int fileId);
+    void readFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockHandle);
+    void markPage(PageIdType pageId);
     File* getHandle(int fileId);
-    
+    void printBitMap(unsigned bm);
+    int mapDiff(unsigned bm1, unsigned bm2);
+	    
     bool _isReg[FILMNG_MAXDATAFILE];
     int _tabSetId[FILMNG_MAXDATAFILE];
     int _fileType[FILMNG_MAXDATAFILE];
     int _numPages[FILMNG_MAXDATAFILE];
-
-    unsigned long long _currentPageOffset;
-    unsigned long long _regPageOffset[FILMNG_MAXDATAFILE];
-
+    PageIdType _pageOffset[FILMNG_MAXDATAFILE];
     File* _fhList[FILMNG_MAXDATAFILE];
-
     Chain _path[FILMNG_MAXDATAFILE];
-
     unsigned* _buMask[FILMNG_MAXDATAFILE];
+    unsigned* _commitMask[FILMNG_MAXDATAFILE];
     bool _fbmMask[FILMNG_MAXDATAFILE];
-
-    int _appendPos[FILMNG_MAXDATAFILE];
-
+    PageIdType _appendPos[FILMNG_MAXDATAFILE];
+    int _appendFid;
+    bool _isReadOnly;
+    
     unsigned long _modId;
 };
 #endif
