@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -33,50 +33,36 @@ class CegoKeyObject : public CegoDecodableObject {
 
 public:
 
-    CegoKeyObject();
-    
-    CegoKeyObject(const CegoKeyObject& ko);
-    
+    CegoKeyObject();    
+    CegoKeyObject(const CegoKeyObject& ko);    
     CegoKeyObject(ObjectType type,
 		  const Chain& objName,
 		  int tabSetId = 0);
-
     CegoKeyObject(int tabSetId,
 		  const Chain& fkey, 
 		  const Chain& tabName, 
 		  const ListT<CegoField>& schemaList,
 		  const Chain& refTable,
 		  const ListT<CegoField>& refList);	
-
     ~CegoKeyObject();
 
     void setTabName(const Chain& name);
     const Chain& getTabName() const;
-    
     void setKeySchema(const ListT<CegoField>& keySchema);   
     const ListT<CegoField>& getKeySchema() const;
-
     void setRefTable(const Chain& refTable);
     const Chain& getRefTable() const;
-    
     void setRefSchema(const ListT<CegoField>& refSchema);   
     const ListT<CegoField>& getRefSchema() const;
-
     int getEntrySize() const;
-
-    void encode(char *buf);
+    void encode(char *buf) const;
     void decode(char *buf);
-
     void setEmpty();
+    Chain toChain() const;
+    Chain getFormatted() const;
     
     CegoKeyObject& operator = ( const CegoKeyObject& oe);
     bool operator == ( const CegoKeyObject& oe);
-    
-    Chain toChain() const;
-
-    Element* getElement() const;
-    void putElement(Element* pElement);
-    Chain getFormatted() const;
 
 private:
 
@@ -84,10 +70,6 @@ private:
 
     Chain _tabName;
     Chain _refTable;
-
-    char _keyLen;
-    char _refLen;
-
     ListT<CegoField> _keySchema;
     ListT<CegoField> _refSchema;
 };
