@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -21,6 +21,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 
 // CEGO INCLUDES
+#include "CegoDefs.h"
 #include "CegoLockHandler.h"
 #include "CegoObjectManager.h"
 #include "CegoDataPointer.h"
@@ -35,15 +36,14 @@ public:
     CegoObjectCursor(CegoBufferPool *pOM, CegoLockHandler *pLockHandle, 
 		     int tabSetId, 
 		     CegoObject::ObjectType type,
-		     int fileId, int pageId);
+		     PageIdType pageId);
     ~CegoObjectCursor();
-    
+
+    void setLastPageSync(bool doSync);
     void* getFirst(int& len, CegoDataPointer& dp);
     void* getNext(int& len, CegoDataPointer& dp);
-    
     void abort();
     void reset();
-
     int getTabSetId();
 
  private:
@@ -51,23 +51,14 @@ public:
     CegoBufferPool* _pBufPool;
     CegoLockHandler* _pLockHandle;
     CegoBufferPage _bp;
-    
-
     CegoObject::ObjectType _type;
     int _tabSetId;
-    int _fileId;
-    int _pageId;
-
+    PageIdType _pageId;
     unsigned long _lockId;
     unsigned long _recLock;
-
     bool _isEOC;
-
-    int _startFileId;
-    int _startPageId;
-
-
+    PageIdType _startPageId;
+    bool _lastPageSync;
     unsigned long _modId;
 };
-
 #endif
