@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,6 +22,7 @@
 
 // LFC INCLUDES
 #include <lfcbase/Chain.h>
+
 // CEGO INCLUDES
 #include "CegoProcStmt.h"
 #include "CegoExpr.h"
@@ -34,24 +35,23 @@ public:
     enum AssignmentMode { EXPR2PARAM, EXPR2NULL };
     
     CegoProcAssignStmt(CegoExpr *pExpr,
-		    CegoProcBlock *pBlock);
-    
+		       CegoProcBlock *pBlock);
     CegoProcAssignStmt(const Chain& paramName,
 		       CegoExpr *pExpr,
-		    CegoProcBlock *pBlock);
-    
+		       CegoProcBlock *pBlock);
     ~CegoProcAssignStmt();
     
     CegoException execute();
-
-    Chain toChain(const Chain& indent) const;
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
     
 private:
-
+    
     AssignmentMode _mode;
     Chain _paramName;
     CegoExpr *_pExpr;
-
+    CegoProcBlock *_pBlock;
 };
 #endif
