@@ -6,7 +6,7 @@
 //               
 // Design and Implementation by Bjoern Lemke
 //                                   
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -41,7 +41,6 @@ void CegoProcBlockStmt::setBlock(CegoProcBlock * pBlock)
 
 CegoException CegoProcBlockStmt::execute()
 {
-
     CegoException result;
     result =  _pBlock->execute();
     if ( result != NONE_EXCEP )
@@ -50,17 +49,22 @@ CegoException CegoProcBlockStmt::execute()
     return NONE_EXCEP;
 }
 
-Chain CegoProcBlockStmt::toChain(const Chain& indent) const
+void CegoProcBlockStmt::cleanUp()
+{
+    _pBlock->cleanUp();
+}
+
+Chain CegoProcBlockStmt::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
     s = indent + Chain("begin\n");
-    s += _pBlock->toChain(indent + indent);
+    s += _pBlock->toChain(defTabSetId, indent + Chain(DEFAULTINDENT));
 
     CegoProcException **pException = _pBlock->getExceptionList().First();
     while ( pException ) 
     {
-	s += indent + Chain("exception when ") + (*pException)->toChain() + Chain(" then \n");	
-	s += (*pException)->getBlock()->toChain(indent + indent);
+	s += indent + Chain("exception when ") + (*pException)->toChain(defTabSetId) + Chain(" then \n");	
+	s += (*pException)->getBlock()->toChain(defTabSetId, indent + Chain(DEFAULTINDENT));
 	pException = _pBlock->getExceptionList().Next();
     }
 
@@ -72,3 +76,17 @@ Chain CegoProcBlockStmt::dbFormat(CegoDatabaseFormater *pForm) const
 {
     return pForm->formatProcBlockStmt(_pBlock);
 }
+
+bool CegoProcBlockStmt::isStatic() const
+{
+    ListT<CegoProcStmt*> stmtList = _pBlock->getStmtList();
+	
+    CegoProcStmt **pStmt = stmtList.First();
+    while ( pStmt )
+    {
+	if ( (*pStmt)->isStatic() == false )
+	    return false;
+	pStmt = stmtList.Next();	
+    }
+    return true;
+}
