@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -21,6 +21,7 @@
 // cego includes
 #include "CegoProcVar.h"
 #include "CegoDatabaseFormater.h"
+#include "CegoTypeConverter.h"
 
 CegoProcVar::CegoProcVar()
 {
@@ -31,12 +32,13 @@ CegoProcVar::CegoProcVar(const Chain& name)
     _name = name;
 }
 
-CegoProcVar::CegoProcVar(const Chain& name, CegoProcVar::VarType paramType, CegoDataType dt, int dtlen, const CegoFieldValue& fv)
+CegoProcVar::CegoProcVar(const Chain& name, CegoProcVar::VarType paramType, CegoDataType dt, int dtlen, int dtdim, const CegoFieldValue& fv)
 {
     _name = name;
     _paramType = paramType;
     _dt = dt;
     _dtlen = dtlen;
+    _dtdim = dtdim;
     _fv = fv;
 }
 
@@ -74,9 +76,26 @@ int CegoProcVar::getLength() const
     return _dtlen;
 }
 
+int CegoProcVar::getDim() const
+{
+    return _dtdim;
+}
+
 void CegoProcVar::setValue(const CegoFieldValue& fv)
 {
     _fv = fv;
+
+    // just if datatype for param has setup, we have to cast
+    if ( _dt != NULL_TYPE )
+    {
+	if ( _fv.castTo(_dt, _dtdim) == false )
+	{
+	    throw Exception(EXLOC, "Cannot cast parameter value from type "
+			    + CegoTypeConverter::getTypeString( fv.getType())
+			    + " to "
+			    + CegoTypeConverter::getTypeString( _dt));
+	}
+    }
 }
 
 const CegoFieldValue& CegoProcVar::getValue() const
@@ -90,6 +109,7 @@ CegoProcVar& CegoProcVar::operator = ( const CegoProcVar& pp)
     _paramType = pp._paramType;
     _dt = pp._dt;
     _dtlen = pp._dtlen;
+    _dtdim = pp._dtdim;
     _fv = pp._fv;
     return (*this);
 }
@@ -140,14 +160,7 @@ Chain CegoProcVar::toChain() const
     case DECIMAL_TYPE:
     {
 	s += Chain(" decimal(");
-	s += Chain(_dtlen);
-	s += Chain(")");
-	break;
-    }
-    case FIXED_TYPE:
-    {
-	s += Chain(" fixed(");
-	s += Chain(_dtlen);
+	s += Chain(_dtlen) + Chain(",") + Chain(_dtdim);
 	s += Chain(")");
 	break;
     }
@@ -198,14 +211,15 @@ Chain CegoProcVar::toChain() const
 	s += Chain(" clob");
 	break;
     }
-
+    case PAGEID_TYPE:
+	break;
     }
     return s;
 }
 
 Chain CegoProcVar::dbFormat(CegoDatabaseFormater *pForm) const
 {
-    return pForm->formatProcVar(_name, _paramType, _dt, _dtlen);
+    return pForm->formatProcVar(_name, _paramType, _dt, _dtlen, _dtdim);
 }
 
 ostream& operator << (ostream& s, const CegoProcVar& pp)
