@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,6 +22,7 @@
 
 // base includes
 #include <lfcbase/ListT.h>
+#include <lfcbase/SetT.h>
 #include <lfcbase/Chain.h>
 
 // cego includes
@@ -39,45 +40,66 @@ public:
     
     CegoProcedure();
     CegoProcedure(const Chain& procName, CegoProcBlock *pBlock);
-    CegoProcedure(const Chain& procName, CegoProcBlock *pBlock, CegoDataType returnType, int returnTypeLen);
-
+    CegoProcedure(const Chain& procName, CegoProcBlock *pBlock, CegoDataType returnType, int returnTypeLen, int returnTypeDim);
     ~CegoProcedure();
 
     void setMasterBlock(CegoProcBlock *pBlock);
-
+    CegoProcBlock* getMasterBlock() const;
     void getArgList(ListT<CegoProcVar>& argList);
-
     ListT<CegoProcVar>& getOutParamList();
-
     const Chain& getName() const;
     CegoProcBlock* getBlock();
-
-    void execute(const ListT<CegoExpr*>& expList);
- 
+    void execute(const ListT<CegoFieldValue>& fvl);
+    void cleanUp();
     CegoFieldValue getRetVal() const;
-
     CegoDataType getReturnType() const;
     int getReturnTypeLen() const;
-
+    int getReturnTypeDim() const;
     ProcType getProcType() const;
-
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    Chain toChain(int defTabSetId) const;
+    void enableProcCache(bool isEnabled);
+    
     CegoProcedure& operator = (const CegoProcedure& p);
     bool operator == (const CegoProcedure& p) const;
+    
+private:
 
-    Chain dbFormat(CegoDatabaseFormater *pForm) const;
-    Chain toChain() const;
+    class ProcCacheValue
+    {
+    public:
+	ProcCacheValue(const ListT<CegoFieldValue>& fvl, const CegoFieldValue& retVal);
+	ProcCacheValue(const ListT<CegoFieldValue>& fvl);
+	ProcCacheValue();
+	~ProcCacheValue();
 
- private:
+	ProcCacheValue& operator = (const ProcCacheValue& pcv);
+	bool operator == (const ProcCacheValue& pcv) const;
+
+	CegoFieldValue& getRetVal();
+	
+    private:
+
+	Chain _paramHash;
+	CegoFieldValue _retVal;
+    };
+
+    bool getCacheValue(const ListT<CegoFieldValue>& fvl, CegoFieldValue& fv);
+    void addCacheValue(const ListT<CegoFieldValue>& fvl, const CegoFieldValue& retVal);
+
+    bool _cacheEnabled;
+    
+    SetT<ProcCacheValue> _cacheSet;
     
     CegoDataType _returnType;
     int _returnTypeLen;
+    int _returnTypeDim;
+    
     ProcType _procType;
     Chain _procName;
     CegoProcBlock* _pBlock;
     CegoProcBlock* _pMasterBlock;
 
     ListT<CegoProcVar> _outParamList;
-
 };
-
 #endif
