@@ -329,7 +329,7 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	}
 	catch ( Exception e )
 	{
-	    throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple"), e);    
+	    throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot map schema for insert"), e);    
 	}
 	
 	if (  execSchema.isEmpty() )
@@ -492,7 +492,7 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		    pExpr = addExprList.Next();
 		}
 	    		
-		throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple"), e);
+		throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple data"), e);
 	    }	       
 	}
 	catch ( CegoQueryException e )
@@ -523,7 +523,7 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		pF = execSchema.Next();
 	    }
 
-	    throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple"), e);	   
+	    throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot execute insert"), e);	   
 	}
 	
 	return Chain(_affCount) + Chain(" tuples inserted");
@@ -681,7 +681,7 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		    }
 		    catch ( Exception e )
 		    {
-			throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple"), e);
+			throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple data"), e);
 		    }
 		}
 	    }
@@ -717,7 +717,7 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		    pFLA=fva.Next();
 		}
 
-		throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot insert tuple"), e);
+		throw CegoQueryException(COREOP_EXCEP, EXLOC, Chain("Cannot execute insert by select"), e);
 	    }
 	    
 	    if ( moreTuple )
