@@ -32,6 +32,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+int __dateLength;
+
 CegoQueryHelper::CegoQueryHelper()
 {
 }
@@ -77,6 +79,12 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 
 	if ( pAttrDesc1 && pAttrDesc2 )
 	{
+
+	    if ( ac.isAnd() == false )
+	    {
+		ac.clean();
+		ac.setAnd(true);
+	    }
 	    
 	    CegoField* pF = schema.First();
 	    
@@ -101,6 +109,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 						    pF->getAttrName(), 
 						    pP->getComparison(),
 						    *pAttrDesc2));
+				return COMPLETE;
 			    }
 			    pFB = flArray[i].Next();
 			}
@@ -137,6 +146,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 						    pF->getAttrName(), 
 						    comp,
 						    *pAttrDesc1));
+				return COMPLETE;
 			    }
 			    pFB = flArray[i].Next();
 			}
@@ -144,9 +154,17 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		}
 		pF = schema.Next();
 	    }
+	    return PARTIAL;
 	}
-	else if ( pAttrDesc1 && attrRefList2.Size() == 0 )
+	else if ( pAttrDesc1 && attrRefList2.Size() == 0  )
 	{
+
+	    if ( ac.isAnd() == false )
+	    {
+		ac.clean();
+		ac.setAnd(true);
+	    }
+
 	    CegoField* pF = schema.First();   
 	    while (pF)
 	    { 
@@ -155,7 +173,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		     pAttrDesc1->getTableName() == Chain())
 		    && pAttrDesc1->getAttrName() == (Chain)pF->getAttrName())
 		{
-		    // pP->getExpr2()->setBlock(pBlock);
 		    ac.add(CegoAttrComp(pF->getTableAlias(), 
 					pF->getAttrName(),
 					pP->getComparison(),
@@ -166,7 +183,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 	    }
 	    return PARTIAL;
 	}
-	else if ( pAttrDesc2 && attrRefList1.Size() == 0 )    
+	else if ( pAttrDesc2 && attrRefList1.Size() == 0 && ac.isAnd() )
 	{
 	    CegoField* pF = schema.First();	    
 	    while (pF)
@@ -176,8 +193,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		     pAttrDesc2->getTableName() == Chain())
 		    && pAttrDesc2->getAttrName() == (Chain)pF->getAttrName())
 		{
-		    // pP->getExpr1()->setBlock(pBlock);
-
 		    // we have to map comparison 
 		    CegoComparison comp = pP->getComparison();
 		    if ( comp == LESS_THAN )
@@ -214,8 +229,16 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 	ListT<CegoAttrDesc*> attrRefList2 = pP->getExpr2()->getAttrRefList();
 	ListT<CegoAttrDesc*> attrRefList3 = pP->getExpr3()->getAttrRefList();
 
-	if ( pAttrDesc1 )
+	if ( pAttrDesc1  )
 	{
+
+	    if ( ac.isAnd() == false )
+	    {
+		ac.clean();
+		ac.setAnd(true);
+	    }
+
+	    
 	    CegoField* pF = schema.First(); 
 	    while (pF)
 	    {		
@@ -226,9 +249,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		{
 		    if (  attrRefList2.Size() == 0 && attrRefList3.Size() == 0 ) 
                     { 
-                        // pP->getExpr2()->setBlock(pBlock); 
-                        // pP->getExpr3()->setBlock(pBlock); 
-
                         ac.add(CegoAttrComp(pF->getTableAlias(), 
                                             pF->getAttrName(), 
                                             pP->getExpr2()->evalFieldValue(0, pBlock), 
@@ -236,8 +256,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
                     } 
                     else if ( pAttrDesc2 && attrRefList3.Size() == 0 ) 
                     { 			
-                        // pP->getExpr3()->setBlock(pBlock); 
-
                         ac.add(CegoAttrComp(pF->getTableAlias(), 
                                             pF->getAttrName(), 
                                             *pAttrDesc2, 
@@ -245,8 +263,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
                     } 
                     else if (  attrRefList2.Size() == 0 &&  pAttrDesc3  ) 
                     { 
-                        // pP->getExpr2()->setBlock(pBlock); 
-
                         ac.add(CegoAttrComp(pF->getTableAlias(), 
                                             pF->getAttrName(), 
                                             pP->getExpr2()->evalFieldValue(0, pBlock), 
@@ -279,7 +295,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
     {
 	CegoAttrDesc* pAttrDesc;	
 	pAttrDesc = pP->getExpr1()->checkAttr();
-	if ( pAttrDesc  )
+	if ( pAttrDesc && ac.isAnd() )
 	{
 	    CegoField* pF = schema.First(); 
 	    while (pF)
@@ -289,7 +305,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		     pAttrDesc->getTableName() == Chain())
 		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
 		{
-		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), false));
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), false, true));
 		    return COMPLETE;
 		}
 		pF = schema.Next();
@@ -305,7 +321,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
     {
 	CegoAttrDesc* pAttrDesc;	
 	pAttrDesc = pP->getExpr1()->checkAttr();
-	if ( pAttrDesc  )
+	if ( pAttrDesc && ac.isAnd() )
 	{
 	    CegoField* pF = schema.First(); 
 	    while (pF)
@@ -315,7 +331,59 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		     pAttrDesc->getTableName() == Chain())
 		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
 		{
-		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), true));
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), true, true));
+		    return COMPLETE;
+		}
+		pF = schema.Next();
+	    }
+	    return PARTIAL;
+	}
+	else
+	{
+	    return PARTIAL;
+	}
+    }
+    else if (pP->getMode() == CegoPredicate::ISNCLIKE)
+    {
+	CegoAttrDesc* pAttrDesc;	
+	pAttrDesc = pP->getExpr1()->checkAttr();
+	if ( pAttrDesc && ac.isAnd() )
+	{
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc->getTableName() == Chain())
+		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+		{
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), false, false));
+		    return COMPLETE;
+		}
+		pF = schema.Next();
+	    }
+	    return PARTIAL;
+	}
+	else
+	{
+	    return PARTIAL;
+	}
+    }
+    else if (pP->getMode() == CegoPredicate::ISNOTNCLIKE)
+    {
+	CegoAttrDesc* pAttrDesc;	
+	pAttrDesc = pP->getExpr1()->checkAttr();
+	if ( pAttrDesc && ac.isAnd() )
+	{
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc->getTableName() == Chain())
+		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+		{
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), true, false));
 		    return COMPLETE;
 		}
 		pF = schema.Next();
@@ -331,7 +399,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
     {
 	CegoAttrDesc* pAttrDesc;	
 	pAttrDesc = pP->getExpr1()->checkAttr();
-	if ( pAttrDesc  )
+	if ( pAttrDesc && ac.isAnd() )
 	{
 	    CegoField* pF = schema.First(); 
 	    while (pF)
@@ -358,7 +426,7 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
     {
 	CegoAttrDesc* pAttrDesc;	
 	pAttrDesc = pP->getExpr1()->checkAttr();
-	if ( pAttrDesc  )
+	if ( pAttrDesc && ac.isAnd() )
 	{
 	    CegoField* pF = schema.First(); 
 	    while (pF)
@@ -387,7 +455,114 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
     }
     else if (pP->getMode() == CegoPredicate::IN)
     {
-	return PARTIAL;	    
+
+	CegoAttrDesc* pAttrDesc1 = pP->getExpr1()->checkAttr();
+	ListT<CegoAttrDesc*> attrRefList1 = pP->getExpr1()->getAttrRefList();
+	
+	if ( pAttrDesc1 && ( ac.numComp() == 0 || ac.hasIndexCandidate() == false ) )
+	{
+	    ac.clean();
+	    ac.setAnd(false);
+	    
+	     
+	    Chain tabAlias;
+	    Chain attrName;
+	    
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc1->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc1->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc1->getTableName() == Chain())
+		    && pAttrDesc1->getAttrName() == (Chain)pF->getAttrName())
+		{		    
+		    tabAlias = pF->getTableAlias();
+		    attrName = pF->getAttrName();
+		}
+		
+		pF = schema.Next(); 
+		
+	    }
+	    
+	    if ( tabAlias == Chain() )
+		return PARTIAL;
+	    
+	    
+	    CegoExpr** pExpr = pP->getExprList().First();
+	    while ( pExpr )
+	    {
+		
+		CegoAttrDesc* pInAttrDesc = (*pExpr)->checkAttr();
+		ListT<CegoAttrDesc*> inAttrRefList = (*pExpr)->getAttrRefList();
+
+		if ( pInAttrDesc )
+		{
+		    bool attrFound=false;
+		    
+		    CegoField* pF = schema.First(); 
+		    while (pF)
+		    {
+			
+			if ((pInAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+			     pInAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+			     pInAttrDesc->getTableName() == Chain())
+			    && pInAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+			{		    
+			    ac.add(CegoAttrComp(tabAlias,
+						attrName,
+						EQUAL,	
+						*pInAttrDesc));
+			    attrFound=true;
+			    
+			}
+			pF = schema.Next();
+		    }
+
+		    if ( attrFound == false )
+		    {
+			for ( int i=0; i<flSize; i++)
+			{
+			    CegoField* pFB = flArray[i].First();
+			    while (pFB)
+			    {
+				
+				if ( (Chain)pInAttrDesc->getTableName() == (Chain)pFB->getTableName() ||
+				     (Chain)pInAttrDesc->getTableName() == (Chain)pFB->getTableAlias() ||
+				     pInAttrDesc->getTableName() == Chain()
+				     && (Chain)pInAttrDesc->getAttrName() == (Chain)pFB->getAttrName())
+				{				
+				    ac.add( CegoAttrComp(tabAlias, 
+							 attrName,
+							 EQUAL,
+							 *pInAttrDesc));
+				    attrFound=true;
+				}
+				pFB = flArray[i].Next();
+			    }
+			}
+		    }
+
+		    if ( attrFound == false )
+			return INAPP;
+
+		}
+		else if ( inAttrRefList.Size() == 0 )		    
+		{
+		    ac.add(CegoAttrComp(tabAlias, 
+					attrName, 
+					EQUAL,
+					(*pExpr)->evalFieldValue(0, pBlock)));
+		}
+		
+		pExpr = pP->getExprList().Next();
+	    }
+	    
+	    return COMPLETE;
+	}
+	else
+	{
+	    return PARTIAL;
+	}
     }
     else if (pP->getMode() == CegoPredicate::NOTIN)
     {
@@ -419,7 +594,8 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCondbyCondition(CegoAttr
 									ListT<CegoField>* flArray, 
 									int flSize, 
 									CegoProcBlock* pBlock)
-{    
+{
+    
     if ( pC->getCondType() == CegoCondition::OR)
     {
 	return INAPP;	
@@ -618,8 +794,6 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
 	
 	if ( pAE )
 	{
-	    // pAE->setFieldListArray( &aggTuple );
-
 	    ListT<CegoField>* fl[2];
 	    fl[0] = &aggTuple;
 	    fl[1] = 0;
@@ -629,7 +803,7 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
 
 	if ( fv.getType() == VARCHAR_TYPE )
 	{
-	    Chain msg = Chain("Aggregation not supported on datatype <") + CEGO_TYPE_MAP[ (int) fv.getType()] + Chain(">");
+	    Chain msg = Chain("sum/avg not supported on datatype <") + CEGO_TYPE_MAP[ (int) fv.getType()] + Chain(">");
 	    throw Exception(EXLOC, msg);
 	}
 	
@@ -651,8 +825,6 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
 	
 	if ( pAE )
 	{
-	    // pAE->setFieldListArray( &aggTuple );
-
 	    ListT<CegoField>* fl[2];
 	    fl[0] = &aggTuple;
 	    fl[1] = 0;
@@ -661,10 +833,8 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
 	}
 	
 	if ( pAgg->getFieldValue().getValue() == 0 )
-	    // if ( pAgg->isInit() )
 	{
 	    pAgg->setFieldValue( fv.getLocalCopy() );
-	    // pAgg->setInit(false);
 	}
 	else
 	{		
@@ -686,7 +856,6 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
 	    fl[0] = &aggTuple;
 	    fl[1] = 0;
 
-	    // pAE->setFieldListArray( &aggTuple );
 	    fv = pAE->evalFieldValue(fl, 0);
 	}	
 	
@@ -828,12 +997,12 @@ bool CegoQueryHelper::checkIndexForPredicate(int tabSetId, CegoDistManager* pGTM
 	    return false;
 	}
     }
-    else if (pPred->getMode() == CegoPredicate::EXPRCOMP)
+    else if ( pPred->getMode() == CegoPredicate::EXPRCOMP  || pPred->getMode() == CegoPredicate::IN )
     {
-	
+
 	SetT<Chain> tableRefSet = pPred->getTableRefSet();
 	ListT<CegoAttrDesc*> attrRefList = pPred->getAttrRefList();
-	
+		
 	if ( tableRefSet.Size() == 1 )
 	{
 	    Chain *pTableName = tableRefSet.First();
@@ -858,6 +1027,7 @@ bool CegoQueryHelper::checkIndexForPredicate(int tabSetId, CegoDistManager* pGTM
 		    tsId = (*pCO)->getTabSetId();
 		    tabName = (*pCO)->getTabName();
 		    found=true;
+
 		}
 		else
 		{
@@ -889,6 +1059,7 @@ bool CegoQueryHelper::checkIndexForPredicate(int tabSetId, CegoDistManager* pGTM
 			    while ( pAttrDesc )
 			    {
 				Chain attrName =  (*pAttrDesc)->getAttrName();
+				
 				CegoAttrAlias *pA = aliasList.First();
 				bool found=false;
 				while ( pA && ! found )
@@ -2150,7 +2321,7 @@ int CegoQueryHelper::maxFieldSize(CegoField *pF)
 	maxLen = max(pF->getAttrName().length(), 
 		     pF->getTableAlias().length(), 
 		     pF->getTableName().length(),
-		     MAX_DATETIME_LEN);
+		     __dateLength);
 	break;
     }
     case FLOAT_TYPE:
