@@ -126,7 +126,7 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, l
 	    pDoc->setRootElement(0);
 	    delete pDoc;
 
-	    Chain buId = pRoot->getAttributeValue(XML_BUBRANCH_ATTR);
+	    Chain buId = pRoot->getAttributeValue(XML_BRANCHID_ATTR);
 	    Chain currentId = _pDBMng->getTableSetBackupBranch(tableSet);
 
 	    if ( buId == currentId )
@@ -227,8 +227,7 @@ unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, l
 	if ( pit != 0 )
 	{
 	    // with a point-in-time recovery, we need new online backup
-	    Datetime d;
-	    _pDBMng->setTableSetBackupBranch(tableSet, d.asChain(BUBRANCHFORMAT));
+	    _pDBMng->nextTableSetBackupBranch(tableSet);
 	}
 	
 	if ( pAH ) 
