@@ -191,7 +191,7 @@ public:
     void initDoc();
     void setTSRoot(const Chain& tableSet, const Chain& tsRoot);
     Chain getTSRoot(const Chain& tableSet);    
-    void setTableSetBackupBranch(const Chain& tableSet, const Chain& branchId);
+    void nextTableSetBackupBranch(const Chain& tableSet);
     Chain getTableSetBackupBranch(const Chain& tableSet);
     void setTSTicket(const Chain& tableSet, const Chain& tsTicket);
     Chain getTSTicket(const Chain& tableSet);
