@@ -332,7 +332,7 @@
 #define XML_AGG_ATTR "AGG"
 #define XML_APPSIZE_ATTR "APPSIZE"
 #define XML_BACKUPMNGPROG_ATTR "BACKUPMNGPROG"
-#define XML_BUBRANCH_ATTR "BUBRANCH"
+#define XML_BRANCHID_ATTR "BRANCHID"
 #define XML_APPENDMODE_ATTR "APPENDMODE"
 #define XML_ARCHMODE_ATTR "ARCHMODE"
 #define XML_ARCHID_ATTR "ARCHID"
